@@ -2,40 +2,39 @@
 """Dark matter spectra."""
 import numpy as np
 import astropy.units as u
-from astropy.units import Quantity
 from astropy.table import Table
-from ...utils.scripts import make_path
-from ...utils.fitting import Parameter
-from ...spectrum.models import SpectralModel, TableModel
+from gammapy.modeling import Parameter
+from gammapy.modeling.models import SpectralModel, TemplateSpectralModel
+from gammapy.utils.scripts import make_path
 
-__all__ = ["PrimaryFlux", "DMAnnihilation"]
+__all__ = ["PrimaryFlux", "DarkMatterAnnihilationSpectralModel"]
 
 
 class PrimaryFlux:
     """DM-annihilation gamma-ray spectra.
 
-    Based on the precomputed models by `Cirelli et al.
-    <http://www.marcocirelli.net/PPPC4DMID.html>`_. All available
+    Based on the precomputed models by Cirelli et al. (2016). All available
     annihilation channels can be found there. The dark matter mass will be set
     to the nearest available value. The spectra will be available as
-    `~gammapy.spectrum.models.TableModel` for a chosen dark matter mass and
+    `~gammapy.modeling.models.TemplateSpectralModel` for a chosen dark matter mass and
     annihilation channel.
 
     References
     ----------
-    * `2011JCAP...03..051 <http://adsabs.harvard.edu/abs/2011JCAP...03..051>`_
+    * `2011JCAP...03..051 <https://ui.adsabs.harvard.edu/abs/2011JCAP...03..051>`_
+    * Cirelli et al (2016): http://www.marcocirelli.net/PPPC4DMID.html
     """
 
     channel_registry = {
         "eL": "eL",
         "eR": "eR",
         "e": "e",
-        "muL": "\\[Mu]L",
-        "muR": "\\[Mu]R",
-        "mu": "\\[Mu]",
-        "tauL": "\\[Tau]L",
-        "tauR": "\\[Tau]R",
-        "tau": "\\[Tau]",
+        "muL": r"\[Mu]L",
+        "muR": r"\[Mu]R",
+        "mu": r"\[Mu]",
+        "tauL": r"\[Tau]L",
+        "tauR": r"\[Tau]R",
+        "tau": r"\[Tau]",
         "q": "q",
         "c": "c",
         "b": "b",
@@ -47,14 +46,14 @@ class PrimaryFlux:
         "ZT": "ZT",
         "Z": "Z",
         "g": "g",
-        "gamma": "\\[Gamma]",
+        "gamma": r"\[Gamma]",
         "h": "h",
-        "nu_e": "\\[Nu]e",
-        "nu_mu": "\\[Nu]\\[Mu]",
-        "nu_tau": "\\[Nu]\\[Tau]",
+        "nu_e": r"\[Nu]e",
+        "nu_mu": r"\[Nu]\[Mu]",
+        "nu_tau": r"\[Nu]\[Tau]",
         "V->e": "V->e",
-        "V->mu": "V->\\[Mu]",
-        "V->tau": "V->\\[Tau]",
+        "V->mu": r"V->\[Mu]",
+        "V->tau": r"V->\[Tau]",
     }
 
     table_filename = "$GAMMAPY_DATA/dark_matter_spectra/AtProduction_gammas.dat"
@@ -63,14 +62,11 @@ class PrimaryFlux:
 
         self.table_path = make_path(self.table_filename)
         if not self.table_path.exists():
-            message = (
-                "\n\nFile {} not found.\n"
+            raise FileNotFoundError(
+                f"\n\nFile not found: {self.table_filename}\n"
                 "You may download the dataset needed with the following command:\n"
                 "gammapy download datasets --src dark_matter_spectra"
-                "".format(self.table_filename)
             )
-
-            raise FileNotFoundError(message)
         else:
             self.table = Table.read(
                 str(self.table_path),
@@ -90,9 +86,9 @@ class PrimaryFlux:
     @mDM.setter
     def mDM(self, mDM):
         mDM_vals = self.table["mDM"].data
-        mDM_ = Quantity(mDM).to_value("GeV")
+        mDM_ = u.Quantity(mDM).to_value("GeV")
         interp_idx = np.argmin(np.abs(mDM_vals - mDM_))
-        self._mDM = Quantity(mDM_vals[interp_idx], "GeV")
+        self._mDM = u.Quantity(mDM_vals[interp_idx], "GeV")
 
     @property
     def allowed_channels(self):
@@ -107,26 +103,25 @@ class PrimaryFlux:
     @channel.setter
     def channel(self, channel):
         if channel not in self.allowed_channels:
-            msg = "Invalid channel {}\n"
-            msg += "Available: {}\n"
-            raise ValueError(msg.format(channel, self.allowed_channels))
+            raise ValueError(
+                f"Invalid channel: {channel}\nAvailable: {self.allowed_channels}\n"
+            )
         else:
             self._channel = channel
 
     @property
     def table_model(self):
-        """Spectrum as `~gammapy.spectrum.models.TableModel`."""
+        """Spectrum as `~gammapy.modeling.models.TemplateSpectralModel`."""
         subtable = self.table[self.table["mDM"] == self.mDM.value]
         energies = (10 ** subtable["Log[10,x]"]) * self.mDM
         channel_name = self.channel_registry[self.channel]
         dN_dlogx = subtable[channel_name]
         dN_dE = dN_dlogx / (energies * np.log(10))
-
-        return TableModel(energy=energies, values=dN_dE, values_scale="lin")
+        return TemplateSpectralModel(energy=energies, values=dN_dE)
 
 
-class DMAnnihilation(SpectralModel):
-    r"""Spectral model for dark matter annihilation.
+class DarkMatterAnnihilationSpectralModel(SpectralModel):
+    r"""Dark matter annihilation spectral model.
 
     The gamma-ray flux is computed as follows:
 
@@ -143,8 +138,8 @@ class DMAnnihilation(SpectralModel):
         Annihilation channel for `~gammapy.astro.darkmatter.PrimaryFlux`
     scale : float
         Scale parameter for model fitting
-    jfact : `~astropy.units.Quantity`
-        Integrated J-Factor needed when `~gammapy.image.models.SkyPointSource` spatial model is used
+    jfactor : `~astropy.units.Quantity`
+        Integrated J-Factor needed when `~gammapy.modeling.models.PointSpatialModel` spatial model is used
     z: float
         Redshift value
     k: int
@@ -152,19 +147,19 @@ class DMAnnihilation(SpectralModel):
 
     Examples
     --------
-    This is how to instantiate a `DMAnnihilation` model::
+    This is how to instantiate a `DarkMatterAnnihilationSpectralModel` model::
 
         from astropy import units as u
-        from gammapy.astro.darkmatter import DMAnnihilation
+        from gammapy.astro.darkmatter import DarkMatterAnnihilationSpectralModel
 
         channel = "b"
         massDM = 5000*u.Unit("GeV")
         jfactor = 3.41e19 * u.Unit("GeV2 cm-5")
-        modelDM = DMAnnihilation(mass=massDM, channel=channel, jfactor=jfactor)
+        modelDM = DarkMatterAnnihilationSpectralModel(mass=massDM, channel=channel, jfactor=jfactor)
 
     References
     ----------
-    * `2011JCAP...03..051 <http://adsabs.harvard.edu/abs/2011JCAP...03..051>`_
+    * `2011JCAP...03..051 <https://ui.adsabs.harvard.edu/abs/2011JCAP...03..051>`_
     """
 
     __slots__ = ["mass", "channel", "scale", "jfactor", "z", "k", "primary_flux"]
@@ -188,7 +183,7 @@ class DMAnnihilation(SpectralModel):
             scale
             * self.jfactor
             * self.THERMAL_RELIC_CROSS_SECTION
-            * self.primary_flux.evaluate(energy=energy * (1 + self.z), norm=1)
+            * self.primary_flux(energy=energy * (1 + self.z))
             / self.k
             / self.mass
             / self.mass
