@@ -3,30 +3,35 @@
 import warnings
 import numpy as np
 import astropy.units as u
-from astropy.table import Table, Column
+from astropy.table import Column, Table
 from astropy.time import Time
-from ..utils.scripts import make_path
-from ..utils.energy import EnergyBounds
-from ..utils.table import table_standardise_units_inplace
-from ..maps import Map
-from ..spectrum import FluxPoints
-from ..spectrum.models import (
-    PowerLaw,
-    PowerLaw2,
-    ExponentialCutoffPowerLaw3FGL,
-    PLSuperExpCutoff3FGL,
-    LogParabola,
+from gammapy.maps import Map
+from gammapy.modeling.models import (
+    DiskSpatialModel,
+    ExpCutoffPowerLaw3FGLSpectralModel,
+    GaussianSpatialModel,
+    LogParabolaSpectralModel,
+    PointSpatialModel,
+    PowerLaw2SpectralModel,
+    PowerLawSpectralModel,
+    SkyModel,
+    SuperExpCutoffPowerLaw3FGLSpectralModel,
+    SuperExpCutoffPowerLaw4FGLSpectralModel,
+    TemplateSpatialModel,
 )
-from ..image.models import SkyPointSource, SkyGaussian, SkyDisk, SkyDiffuseMap
-from ..cube.models import SkyModel
-from ..time import LightCurve
+from gammapy.spectrum import FluxPoints
+from gammapy.time import LightCurve
+from gammapy.utils.scripts import make_path
+from gammapy.utils.table import table_standardise_units_inplace
 from .core import SourceCatalog, SourceCatalogObject
 
 __all__ = [
+    "SourceCatalogObject4FGL",
     "SourceCatalogObject3FGL",
     "SourceCatalogObject1FHL",
     "SourceCatalogObject2FHL",
     "SourceCatalogObject3FHL",
+    "SourceCatalog4FGL",
     "SourceCatalog3FGL",
     "SourceCatalog1FHL",
     "SourceCatalog2FHL",
@@ -42,13 +47,58 @@ def compute_flux_points_ul(quantity, quantity_errp):
     return 2 * quantity_errp + quantity
 
 
+class SourceCatalogObject4FGL(SourceCatalogObject):
+    """One source from the Fermi-LAT 4FGL catalog.
+
+    Catalog is represented by `~gammapy.catalog.SourceCatalog4FGL`.
+    """
+
+    @property
+    def spectral_model(self):
+        """Best fit spectral model (`~gammapy.modeling.models.SpectralModel`)."""
+        spec_type = self.data["SpectrumType"].strip()
+
+        pars, errs = {}, {}
+        pars["reference"] = self.data["Pivot_Energy"]
+
+        if spec_type == "PowerLaw":
+            pars["amplitude"] = self.data["PL_Flux_Density"]
+            pars["index"] = self.data["PL_Index"]
+            errs["amplitude"] = self.data["Unc_PL_Flux_Density"]
+            errs["index"] = self.data["Unc_PL_Index"]
+            model = PowerLawSpectralModel(**pars)
+        elif spec_type == "LogParabola":
+            pars["amplitude"] = self.data["LP_Flux_Density"]
+            pars["alpha"] = self.data["LP_Index"]
+            pars["beta"] = self.data["LP_beta"]
+            errs["amplitude"] = self.data["Unc_LP_Flux_Density"]
+            errs["alpha"] = self.data["Unc_LP_Index"]
+            errs["beta"] = self.data["Unc_LP_beta"]
+            model = LogParabolaSpectralModel(**pars)
+        elif spec_type == "PLSuperExpCutoff":
+            pars["amplitude"] = self.data["PLEC_Flux_Density"]
+            pars["index_1"] = self.data["PLEC_Index"]
+            pars["index_2"] = self.data["PLEC_Exp_Index"]
+            pars["expfactor"] = self.data["PLEC_Expfactor"]
+            errs["amplitude"] = self.data["Unc_PLEC_Flux_Density"]
+            errs["index_1"] = self.data["Unc_PLEC_Index"]
+            errs["index_2"] = np.nan_to_num(self.data["Unc_PLEC_Exp_Index"])
+            errs["expfactor"] = self.data["Unc_PLEC_Expfactor"]
+            model = SuperExpCutoffPowerLaw4FGLSpectralModel(**pars)
+        else:
+            raise ValueError(f"Invalid spec_type: {spec_type!r}")
+
+        model.parameters.set_parameter_errors(errs)
+        return model
+
+
 class SourceCatalogObject3FGL(SourceCatalogObject):
     """One source from the Fermi-LAT 3FGL catalog.
 
     Catalog is represented by `~gammapy.catalog.SourceCatalog3FGL`.
     """
 
-    _ebounds = EnergyBounds([100, 300, 1000, 3000, 10000, 100000], "MeV")
+    _ebounds = u.Quantity([100, 300, 1000, 3000, 10000, 100000], "MeV")
     _ebounds_suffix = ["100_300", "300_1000", "1000_3000", "3000_10000", "10000_100000"]
     energy_range = u.Quantity([100, 100000], "MeV")
     """Energy range of the catalog.
@@ -144,7 +194,7 @@ class SourceCatalogObject3FGL(SourceCatalogObject):
             " 0.25.",
             10: "Spectral Fit Quality > 16.3 (see Equation 3 in 2FGL catalog paper).",
             11: "Possibly due to the Sun (see Section 3.6 in catalog paper).",
-            12: "Highly curved spectrum; LogParabola beta fixed to 1 or PLExpCutoff Spectral Index fixed to 0 (see "
+            12: "Highly curved spectrum; LogParabolaSpectralModel beta fixed to 1 or PLExpCutoff Spectral Index fixed to 0 (see "
             "Section 3.3 in catalog paper).",
         }
         ss += "{:<20s} : {}\n".format(
@@ -202,7 +252,7 @@ class SourceCatalogObject3FGL(SourceCatalogObject):
                 "Super-exponential cutoff index", d["Exp_Index"], d["Unc_Exp_Index"]
             )
         else:
-            raise ValueError("Invalid spec_type")
+            raise ValueError(f"Invalid spec_type: {spec_type!r}")
 
         ss += "{:<45s} : {:.0f} {}\n".format(
             "Pivot energy", d["Pivot_Energy"].value, d["Pivot_Energy"].unit
@@ -283,7 +333,7 @@ class SourceCatalogObject3FGL(SourceCatalogObject):
 
     @property
     def spectral_model(self):
-        """Best fit spectral model (`~gammapy.spectrum.models.SpectralModel`)."""
+        """Best fit spectral model (`~gammapy.modeling.models.SpectralModel`)."""
         spec_type = self.data["SpectrumType"].strip()
 
         pars, errs = {}, {}
@@ -294,19 +344,19 @@ class SourceCatalogObject3FGL(SourceCatalogObject):
         if spec_type == "PowerLaw":
             pars["index"] = self.data["Spectral_Index"]
             errs["index"] = self.data["Unc_Spectral_Index"]
-            model = PowerLaw(**pars)
+            model = PowerLawSpectralModel(**pars)
         elif spec_type == "PLExpCutoff":
             pars["index"] = self.data["Spectral_Index"]
             pars["ecut"] = self.data["Cutoff"]
             errs["index"] = self.data["Unc_Spectral_Index"]
             errs["ecut"] = self.data["Unc_Cutoff"]
-            model = ExponentialCutoffPowerLaw3FGL(**pars)
+            model = ExpCutoffPowerLaw3FGLSpectralModel(**pars)
         elif spec_type == "LogParabola":
             pars["alpha"] = self.data["Spectral_Index"]
             pars["beta"] = self.data["beta"]
             errs["alpha"] = self.data["Unc_Spectral_Index"]
             errs["beta"] = self.data["Unc_beta"]
-            model = LogParabola(**pars)
+            model = LogParabolaSpectralModel(**pars)
         elif spec_type == "PLSuperExpCutoff":
             # TODO: why convert to GeV here? Remove?
             pars["reference"] = pars["reference"].to("GeV")
@@ -316,9 +366,9 @@ class SourceCatalogObject3FGL(SourceCatalogObject):
             errs["index_1"] = self.data["Unc_Spectral_Index"]
             errs["index_2"] = self.data["Unc_Exp_Index"]
             errs["ecut"] = self.data["Unc_Cutoff"].to("GeV")
-            model = PLSuperExpCutoff3FGL(**pars)
+            model = SuperExpCutoffPowerLaw3FGLSpectralModel(**pars)
         else:
-            raise ValueError("Invalid spec_type: {!r}".format(spec_type))
+            raise ValueError(f"Invalid spec_type: {spec_type!r}")
 
         model.parameters.set_parameter_errors(errs)
         return model
@@ -326,7 +376,7 @@ class SourceCatalogObject3FGL(SourceCatalogObject):
     @property
     def spatial_model(self):
         """
-        Source spatial model (`~gammapy.image.models.SkySpatialModel`).
+        Source spatial model (`~gammapy.modeling.models.SpatialModel`).
         """
         d = self.data
 
@@ -337,30 +387,34 @@ class SourceCatalogObject3FGL(SourceCatalogObject):
         if self.is_pointlike:
             pars["lon_0"] = glon
             pars["lat_0"] = glat
-            return SkyPointSource(lon_0=glon, lat_0=glat)
+            return PointSpatialModel(lon_0=glon, lat_0=glat, frame="galactic")
         else:
             de = self.data_extended
             morph_type = de["Model_Form"].strip()
 
             if morph_type == "Disk":
                 r_0 = de["Model_SemiMajor"].to("deg")
-                return SkyDisk(lon_0=glon, lat_0=glat, r_0=r_0)
+                return DiskSpatialModel(
+                    lon_0=glon, lat_0=glat, r_0=r_0, frame="galactic"
+                )
             elif morph_type in ["Map", "Ring", "2D Gaussian x2"]:
                 filename = de["Spatial_Filename"].strip()
                 path = make_path(
                     "$GAMMAPY_DATA/catalogs/fermi/Extended_archive_v15/Templates/"
                 )
-                return SkyDiffuseMap.read(path / filename)
+                return TemplateSpatialModel.read(path / filename)
             elif morph_type == "2D Gaussian":
                 # TODO: fill elongation info as soon as model supports it
                 sigma = de["Model_SemiMajor"].to("deg")
-                return SkyGaussian(lon_0=glon, lat_0=glat, sigma=sigma)
+                return GaussianSpatialModel(
+                    lon_0=glon, lat_0=glat, sigma=sigma, frame="galactic"
+                )
             else:
-                raise ValueError("Invalid spatial model: {!r}".format(morph_type))
+                raise ValueError(f"Invalid spatial model: {morph_type!r}")
 
     @property
     def sky_model(self):
-        """Source sky model (`~gammapy.cube.models.SkyModel`)."""
+        """Source sky model (`~gammapy.modeling.models.SkyModel`)."""
         spatial_model = self.spatial_model
         spectral_model = self.spectral_model
         return SkyModel(spatial_model, spectral_model, name=self.name)
@@ -374,10 +428,9 @@ class SourceCatalogObject3FGL(SourceCatalogObject):
         """Flux points (`~gammapy.spectrum.FluxPoints`)."""
         table = Table()
         table.meta["SED_TYPE"] = "flux"
-        e_ref = self._ebounds.log_centers
-        table["e_ref"] = e_ref
-        table["e_min"] = self._ebounds.lower_bounds
-        table["e_max"] = self._ebounds.upper_bounds
+
+        table["e_min"] = self._ebounds[:-1]
+        table["e_max"] = self._ebounds[1:]
 
         flux = self._get_flux_values("Flux")
         flux_err = self._get_flux_values("Unc_Flux")
@@ -462,7 +515,7 @@ class SourceCatalogObject1FHL(SourceCatalogObject):
     Catalog is represented by `~gammapy.catalog.SourceCatalog1FHL`.
     """
 
-    _ebounds = EnergyBounds([10, 30, 100, 500], "GeV")
+    _ebounds = u.Quantity([10, 30, 100, 500], "GeV")
     _ebounds_suffix = ["10_30", "30_100", "100_500"]
     energy_range = u.Quantity([0.01, 0.5], "TeV")
     """Energy range of the Fermi 1FHL source catalog"""
@@ -484,10 +537,6 @@ class SourceCatalogObject1FHL(SourceCatalogObject):
         ss += "GLAT        : {}\n".format(d["GLAT"])
         ss += "\n"
 
-        # val, err = d['Energy_Flux100'], d['Unc_Energy_Flux100']
-        # ss += 'Energy flux (100 MeV - 100 GeV) : {} +- {} erg cm^-2 s^-1\n'.format(val, err)
-        # ss += 'Detection significance : {}\n'.format(d['Signif_Avg'])
-
         return ss
 
     def _get_flux_values(self, prefix, unit="cm-2 s-1"):
@@ -499,8 +548,8 @@ class SourceCatalogObject1FHL(SourceCatalogObject):
         """Integral flux points (`~gammapy.spectrum.FluxPoints`)."""
         table = Table()
         table.meta["SED_TYPE"] = "flux"
-        table["e_min"] = self._ebounds.lower_bounds
-        table["e_max"] = self._ebounds.upper_bounds
+        table["e_min"] = self._ebounds[:-1]
+        table["e_max"] = self._ebounds[1:]
         table["flux"] = self._get_flux_values("Flux")
         flux_err = self._get_flux_values("Unc_Flux")
         table["flux_errn"] = np.abs(flux_err[:, 0])
@@ -516,14 +565,14 @@ class SourceCatalogObject1FHL(SourceCatalogObject):
 
     @property
     def spectral_model(self):
-        """Best fit spectral model `~gammapy.spectrum.models.SpectralModel`."""
+        """Best fit spectral model `~gammapy.modeling.models.SpectralModel`."""
         pars, errs = {}, {}
         pars["amplitude"] = self.data["Flux"]
         pars["emin"], pars["emax"] = self.energy_range
         pars["index"] = self.data["Spectral_Index"]
         errs["amplitude"] = self.data["Unc_Flux"]
         errs["index"] = self.data["Unc_Spectral_Index"]
-        model = PowerLaw2(**pars)
+        model = PowerLaw2SpectralModel(**pars)
         model.parameters.set_parameter_errors(errs)
         return model
 
@@ -534,7 +583,7 @@ class SourceCatalogObject2FHL(SourceCatalogObject):
     Catalog is represented by `~gammapy.catalog.SourceCatalog2FHL`.
     """
 
-    _ebounds = EnergyBounds([50, 171, 585, 2000], "GeV")
+    _ebounds = u.Quantity([50, 171, 585, 2000], "GeV")
     _ebounds_suffix = ["50_171", "171_585", "585_2000"]
     energy_range = u.Quantity([0.05, 2], "TeV")
     """Energy range of the Fermi 2FHL source catalog"""
@@ -571,8 +620,8 @@ class SourceCatalogObject2FHL(SourceCatalogObject):
         """Integral flux points (`~gammapy.spectrum.FluxPoints`)."""
         table = Table()
         table.meta["SED_TYPE"] = "flux"
-        table["e_min"] = self._ebounds.lower_bounds
-        table["e_max"] = self._ebounds.upper_bounds
+        table["e_min"] = self._ebounds[:-1]
+        table["e_max"] = self._ebounds[1:]
         table["flux"] = self._get_flux_values("Flux")
         flux_err = self._get_flux_values("Unc_Flux")
         table["flux_errn"] = np.abs(flux_err[:, 0])
@@ -588,7 +637,7 @@ class SourceCatalogObject2FHL(SourceCatalogObject):
 
     @property
     def spectral_model(self):
-        """Best fit spectral model (`~gammapy.spectrum.models.SpectralModel`)."""
+        """Best fit spectral model (`~gammapy.modeling.models.SpectralModel`)."""
         pars, errs = {}, {}
         pars["amplitude"] = self.data["Flux50"]
         pars["emin"], pars["emax"] = self.energy_range
@@ -597,7 +646,7 @@ class SourceCatalogObject2FHL(SourceCatalogObject):
         errs["amplitude"] = self.data["Unc_Flux50"]
         errs["index"] = self.data["Unc_Spectral_Index"]
 
-        model = PowerLaw2(**pars)
+        model = PowerLaw2SpectralModel(**pars)
         model.parameters.set_parameter_errors(errs)
         return model
 
@@ -611,7 +660,7 @@ class SourceCatalogObject3FHL(SourceCatalogObject):
     energy_range = u.Quantity([0.01, 2], "TeV")
     """Energy range of the Fermi 1FHL source catalog"""
 
-    _ebounds = EnergyBounds([10, 20, 50, 150, 500, 2000], "GeV")
+    _ebounds = u.Quantity([10, 20, 50, 150, 500, 2000], "GeV")
 
     def __str__(self):
         return self.info()
@@ -730,16 +779,16 @@ class SourceCatalogObject3FHL(SourceCatalogObject):
         elif spec_type == "LogParabola":
             fmt = "{:<32s} : {:.3f} +- {:.3f}\n"
             ss += fmt.format(
-                "LogParabola spectral index",
+                "LogParabolaSpectralModel spectral index",
                 d["Spectral_Index"],
                 d["Unc_Spectral_Index"],
             )
 
             ss += "{:<32s} : {:.3f} +- {:.3f}\n".format(
-                "LogParabola beta", d["beta"], d["Unc_beta"]
+                "LogParabolaSpectralModel beta", d["beta"], d["Unc_beta"]
             )
         else:
-            raise ValueError("Invalid spec_type")
+            raise ValueError(f"Invalid spec_type: {spec_type!r}")
 
         ss += "{:<32s} : {:.1f} {}\n".format(
             "Pivot energy", d["Pivot_Energy"].value, d["Pivot_Energy"].unit
@@ -802,7 +851,7 @@ class SourceCatalogObject3FHL(SourceCatalogObject):
 
     @property
     def spectral_model(self):
-        """Best fit spectral model (`~gammapy.spectrum.models.SpectralModel`)."""
+        """Best fit spectral model (`~gammapy.modeling.models.SpectralModel`)."""
         d = self.data
         spec_type = self.data["SpectrumType"].strip()
 
@@ -814,15 +863,15 @@ class SourceCatalogObject3FHL(SourceCatalogObject):
         if spec_type == "PowerLaw":
             pars["index"] = d["PowerLaw_Index"]
             errs["index"] = d["Unc_PowerLaw_Index"]
-            model = PowerLaw(**pars)
+            model = PowerLawSpectralModel(**pars)
         elif spec_type == "LogParabola":
             pars["alpha"] = d["Spectral_Index"]
             pars["beta"] = d["beta"]
             errs["alpha"] = d["Unc_Spectral_Index"]
             errs["beta"] = d["Unc_beta"]
-            model = LogParabola(**pars)
+            model = LogParabolaSpectralModel(**pars)
         else:
-            raise ValueError("Invalid spec_type: {!r}".format(spec_type))
+            raise ValueError(f"Invalid spec_type: {spec_type!r}")
 
         model.parameters.set_parameter_errors(errs)
         return model
@@ -832,10 +881,8 @@ class SourceCatalogObject3FHL(SourceCatalogObject):
         """Flux points (`~gammapy.spectrum.FluxPoints`)."""
         table = Table()
         table.meta["SED_TYPE"] = "flux"
-        e_ref = self._ebounds.log_centers
-        table["e_ref"] = e_ref
-        table["e_min"] = self._ebounds.lower_bounds
-        table["e_max"] = self._ebounds.upper_bounds
+        table["e_min"] = self._ebounds[:-1]
+        table["e_max"] = self._ebounds[1:]
 
         flux = self.data["Flux_Band"]
         flux_err = self.data["Unc_Flux_Band"]
@@ -867,7 +914,7 @@ class SourceCatalogObject3FHL(SourceCatalogObject):
 
     @property
     def spatial_model(self):
-        """Source spatial model (`~gammapy.image.models.SkySpatialModel`)."""
+        """Source spatial model (`~gammapy.modeling.models.SpatialModel`)."""
         d = self.data
 
         pars = {}
@@ -877,30 +924,34 @@ class SourceCatalogObject3FHL(SourceCatalogObject):
         if self.is_pointlike:
             pars["lon_0"] = glon
             pars["lat_0"] = glat
-            return SkyPointSource(lon_0=glon, lat_0=glat)
+            return PointSpatialModel(lon_0=glon, lat_0=glat, frame="galactic")
         else:
             de = self.data_extended
             morph_type = de["Spatial_Function"].strip()
 
             if morph_type == "RadialDisk":
                 r_0 = de["Model_SemiMajor"].to("deg")
-                return SkyDisk(lon_0=glon, lat_0=glat, r_0=r_0)
+                return DiskSpatialModel(
+                    lon_0=glon, lat_0=glat, r_0=r_0, frame="galactic"
+                )
             elif morph_type in ["SpatialMap"]:
                 filename = de["Spatial_Filename"].strip()
                 path = make_path(
                     "$GAMMAPY_DATA/catalogs/fermi/Extended_archive_v18/Templates/"
                 )
-                return SkyDiffuseMap.read(path / filename)
+                return TemplateSpatialModel.read(path / filename)
             elif morph_type == "RadialGauss":
                 # TODO: fill elongation info as soon as model supports it
                 sigma = de["Model_SemiMajor"].to("deg")
-                return SkyGaussian(lon_0=glon, lat_0=glat, sigma=sigma)
+                return GaussianSpatialModel(
+                    lon_0=glon, lat_0=glat, sigma=sigma, frame="galactic"
+                )
             else:
-                raise ValueError("Invalid morph_type: {!r}".format(morph_type))
+                raise ValueError(f"Invalid morph_type: {morph_type!r}")
 
     @property
     def sky_model(self):
-        """Source sky model (`~gammapy.cube.models.SkyModel`)."""
+        """Source sky model (`~gammapy.modeling.models.SkyModel`)."""
         spatial_model = self.spatial_model
         spectral_model = self.spectral_model
         return SkyModel(spatial_model, spectral_model, name=self.name)
@@ -987,7 +1038,7 @@ class SourceCatalog3FGL(SourceCatalog):
 
         The classes are described in Table 3 of the 3FGL paper:
 
-        http://adsabs.harvard.edu/abs/2015arXiv150102003T
+        https://ui.adsabs.harvard.edu/abs/2015ApJS..218...23A
 
         Parameters
         ----------
@@ -1020,7 +1071,7 @@ class SourceCatalog3FGL(SourceCatalog):
         elif source_class in np.unique(source_class_info):
             category = {source_class}
         else:
-            raise ValueError("Invalid source_class: {!r}".format(source_class))
+            raise ValueError(f"Invalid source_class: {source_class!r}")
 
         return np.array([_ in category for _ in source_class_info])
 
@@ -1046,10 +1097,51 @@ class SourceCatalog3FGL(SourceCatalog):
         return catalog
 
 
+class SourceCatalog4FGL(SourceCatalog):
+    """Fermi-LAT 4FGL source catalog.
+
+    References:
+
+    - https://arxiv.org/abs/1902.10045
+    - https://fermi.gsfc.nasa.gov/ssc/data/access/lat/8yr_catalog/
+
+    One source is represented by `~gammapy.catalog.SourceCatalogObject4FGL`.
+    """
+
+    name = "4fgl"
+    description = "LAT 8-year point source catalog"
+    source_object_class = SourceCatalogObject4FGL
+
+    def __init__(self, filename="$GAMMAPY_DATA/catalogs/fermi/gll_psc_v19.fit.gz"):
+        filename = str(make_path(filename))
+        table = Table.read(filename, hdu="LAT_Point_Source_Catalog")
+        table_standardise_units_inplace(table)
+
+        source_name_key = "Source_Name"
+        source_name_alias = (
+            "Extended_Source_Name",
+            "ASSOC_FGL",
+            "ASSOC_FHL",
+            "ASSOC_GAM1",
+            "ASSOC_GAM2",
+            "ASSOC_GAM3",
+            "ASSOC_TEV",
+            "ASSOC1",
+            "ASSOC2",
+        )
+        super().__init__(
+            table=table,
+            source_name_key=source_name_key,
+            source_name_alias=source_name_alias,
+        )
+
+        self.extended_sources_table = Table.read(filename, hdu="ExtendedSources")
+
+
 class SourceCatalog1FHL(SourceCatalog):
     """Fermi-LAT 1FHL source catalog.
 
-    Reference: http://adsabs.harvard.edu/abs/2013ApJS..209...34A
+    Reference: https://ui.adsabs.harvard.edu/abs/2013ApJS..209...34A
 
     One source is represented by `~gammapy.catalog.SourceCatalogObject1FHL`.
     """
@@ -1081,7 +1173,7 @@ class SourceCatalog1FHL(SourceCatalog):
 class SourceCatalog2FHL(SourceCatalog):
     """Fermi-LAT 2FHL source catalog.
 
-    Reference: http://adsabs.harvard.edu/abs/2016ApJS..222....5A
+    Reference: https://ui.adsabs.harvard.edu/abs/2016ApJS..222....5A
 
     One source is represented by `~gammapy.catalog.SourceCatalogObject2FHL`.
     """
@@ -1115,7 +1207,7 @@ class SourceCatalog2FHL(SourceCatalog):
 class SourceCatalog3FHL(SourceCatalog):
     """Fermi-LAT 3FHL source catalog.
 
-    Reference: http://adsabs.harvard.edu/abs/2017ApJS..232...18A
+    Reference: https://ui.adsabs.harvard.edu/abs/2017ApJS..232...18A
 
     One source is represented by `~gammapy.catalog.SourceCatalogObject3FHL`.
     """
@@ -1158,7 +1250,7 @@ class SourceCatalog3FHL(SourceCatalog):
 
         The classes are described in Table 3 of the 3FGL paper:
 
-        http://adsabs.harvard.edu/abs/2015arXiv150102003T
+        https://ui.adsabs.harvard.edu/abs/2015ApJS..218...23A
 
         Parameters
         ----------
@@ -1191,7 +1283,7 @@ class SourceCatalog3FHL(SourceCatalog):
         elif source_class in np.unique(source_class_info):
             category = {source_class}
         else:
-            raise ValueError("Invalid source_class: {!r}".format(source_class))
+            raise ValueError(f"Invalid source_class: {source_class!r}")
 
         return np.array([_ in category for _ in source_class_info])
 
