@@ -1,15 +1,29 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from collections import Counter
+import collections
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose
+import astropy
 from astropy import units as u
-from astropy.coordinates import SkyCoord, Angle
+from astropy.coordinates import Angle, SkyCoord
 from astropy.table import Table
-from ...utils.testing import assert_quantity_allclose
-from ...utils.testing import requires_data, requires_dependency
-from ...spectrum.models import PowerLaw, ExponentialCutoffPowerLaw
-from ..hess import SourceCatalogHGPS, SourceCatalogLargeScaleHGPS
+from astropy.utils.data import get_pkg_data_filename
+from gammapy.catalog import SourceCatalogHGPS, SourceCatalogLargeScaleHGPS
+from gammapy.modeling.models import (
+    ExpCutoffPowerLawSpectralModel,
+    PowerLawSpectralModel,
+)
+from gammapy.utils.testing import (
+    assert_quantity_allclose,
+    requires_data,
+    requires_dependency,
+)
+
+SOURCES = [
+    {"idx": 33, "name": "HESS J1713-397", "str_ref_file": "data/hess_j1713-397.txt"},
+    {"idx": 54, "name": "HESS J1825-137", "str_ref_file": "data/hess_j1825-137.txt"},
+    {"idx": 76, "name": "HESS J1930+188", "str_ref_file": "data/hess_j1930+188.txt"},
+]
 
 
 @pytest.fixture(scope="session")
@@ -17,13 +31,17 @@ def cat():
     return SourceCatalogHGPS("$GAMMAPY_DATA/catalogs/hgps_catalog_v1.fits.gz")
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestSourceCatalogHGPS:
     @staticmethod
     def test_source_table(cat):
         assert cat.name == "hgps"
         assert len(cat.table) == 78
 
+    @staticmethod
+    def test_positions(cat):
+        assert len(cat.positions) == 78
+
     @staticmethod
     def test_table_components(cat):
         assert len(cat.table_components) == 98
@@ -48,7 +66,7 @@ class TestSourceCatalogHGPS:
         assert isinstance(cat.large_scale_component, SourceCatalogLargeScaleHGPS)
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestSourceCatalogObjectHGPS:
     @pytest.fixture(scope="class")
     def source(self, cat):
@@ -82,18 +100,20 @@ class TestSourceCatalogObjectHGPS:
         assert "Component HGPSC 083:" in ss
 
     @staticmethod
-    def test_str(cat):
-        source = cat["HESS J1930+188"]
-        assert source.data["Spatial_Model"] == "Gaussian"
-        assert "Spatial components   : HGPSC 097" in str(source)
+    @pytest.mark.skipif(
+        astropy.__version__ <= "3.2", reason="table formatting differences"
+    )
+    @pytest.mark.parametrize("ref", SOURCES)
+    def test_str(cat, ref):
+        actual = str(cat[ref["idx"]])
+        expected = open(get_pkg_data_filename(ref["str_ref_file"])).read()
+        assert actual == expected
 
-        source = cat["HESS J1825-137"]
-        assert source.data["Spatial_Model"] == "3-Gaussian"
-        assert "Spatial components   : HGPSC 065, HGPSC 066, HGPSC 067" in str(source)
-
-        source = cat["HESS J1713-397"]
-        assert source.data["Spatial_Model"] == "Shell"
-        assert "Source name          : HESS J1713-397" in str(source)
+    @staticmethod
+    def test_position(source):
+        position = source.position
+        assert_allclose(position.ra.deg, 280.95162964)
+        assert_allclose(position.dec.deg, -3.55410194)
 
     @staticmethod
     def test_components(source):
@@ -110,7 +130,7 @@ class TestSourceCatalogObjectHGPS:
 
     @staticmethod
     def test_spectral_model_type(cat):
-        spec_types = Counter([_.spectral_model_type for _ in cat])
+        spec_types = collections.Counter([_.spectral_model_type for _ in cat])
         assert spec_types == {"pl": 66, "ecpl": 12}
 
     @staticmethod
@@ -120,7 +140,7 @@ class TestSourceCatalogObjectHGPS:
 
         model = source.spectral_model()
 
-        assert isinstance(model, PowerLaw)
+        assert isinstance(model, PowerLawSpectralModel)
         pars = model.parameters
         assert_allclose(pars["amplitude"].value, 9.140179932365378e-13)
         assert_allclose(pars["index"].value, 2.1513476371765137)
@@ -136,7 +156,7 @@ class TestSourceCatalogObjectHGPS:
         source = cat["HESS J0835-455"]
 
         model = source.spectral_model()
-        assert isinstance(model, ExponentialCutoffPowerLaw)
+        assert isinstance(model, ExpCutoffPowerLawSpectralModel)
 
         pars = model.parameters
         assert_allclose(pars["amplitude"].value, 6.408420542586617e-12)
@@ -149,7 +169,7 @@ class TestSourceCatalogObjectHGPS:
         assert_allclose(err, source.data["Flux_Spec_Int_1TeV_Err"].value, rtol=0.01)
 
         model = source.spectral_model("pl")
-        assert isinstance(model, PowerLaw)
+        assert isinstance(model, PowerLawSpectralModel)
 
         pars = model.parameters
         assert_allclose(pars["amplitude"].value, 1.833056926733856e-12)
@@ -162,7 +182,7 @@ class TestSourceCatalogObjectHGPS:
 
     @staticmethod
     def test_spatial_model_type(cat):
-        morph_types = Counter([_.spatial_model_type for _ in cat])
+        morph_types = collections.Counter([_.spatial_model_type for _ in cat])
         assert morph_types == {
             "gaussian": 52,
             "2-gaussian": 8,
@@ -184,7 +204,7 @@ class TestSourceCatalogObjectHGPS:
         model = cat["HESS J1119-614"].sky_model()
         p = model.parameters
         assert_allclose(p["amplitude"].value, 7.959899015960725e-13)
-        assert_allclose(p["lon_0"].value, -67.871918)
+        assert_allclose(p["lon_0"].value, 292.128082)
         assert_allclose(p["lat_0"].value, -0.5332353711128235)
         assert_allclose(p["sigma"].value, 0.09785966575145721)
 
@@ -241,13 +261,13 @@ class TestSourceCatalogObjectHGPS:
         model = cat["Vela Junior"].sky_model()
         p = model.parameters
         assert_allclose(p["amplitude"].value, 3.2163001428830995e-11)
-        assert_allclose(p["lon_0"].value, -93.712616)
+        assert_allclose(p["lon_0"].value, 266.287384)
         assert_allclose(p["lat_0"].value, -1.243260383605957)
         assert_allclose(p["radius"].value, 0.95)
         assert_allclose(p["width"].value, 0.05)
 
 
-@requires_data("gammapy-data")
+@requires_data()
 class TestSourceCatalogObjectHGPSComponent:
     @pytest.fixture(scope="class")
     def component(self, cat):
@@ -272,6 +292,7 @@ class TestSourceCatalogObjectHGPSComponent:
     @staticmethod
     def test_spatial_model(component):
         model = component.spatial_model
+        assert model.frame == "galactic"
         p = model.parameters
         assert_allclose(p["lon_0"].value, 28.77037811279297)
         assert_allclose(p.error("lon_0"), 0.058748625218868256)
