@@ -1,13 +1,17 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import logging
-from astropy.nddata.utils import NoOverlapError, PartialOverlapError
 from astropy.coordinates import Angle
+from astropy.nddata.utils import NoOverlapError
 from astropy.utils import lazyproperty
-from ..maps import Map, WcsGeom
-from .counts import fill_map_counts
-from .exposure import make_map_exposure_true_energy, _map_spectrum_weight
+from gammapy.irf import EnergyDependentMultiGaussPSF
+from gammapy.maps import Map, WcsGeom
+from gammapy.modeling.models import BackgroundModel
 from .background import make_map_background_irf
-
+from .counts import fill_map_counts
+from .edisp_map import make_edisp_map
+from .exposure import _map_spectrum_weight, make_map_exposure_true_energy
+from .fit import BINSZ_IRF, MIGRA_AXIS_DEFAULT, RAD_AXIS_DEFAULT, MapDataset
+from .psf_map import make_psf_map
 
 __all__ = ["MapMaker", "MapMakerObs", "MapMakerRing"]
 
@@ -28,9 +32,18 @@ class MapMaker:
         If none, the same as geom is assumed
     exclusion_mask : `~gammapy.maps.Map`
         Exclusion mask
+    background_oversampling : int
+        Background oversampling factor in energy axis.
     """
 
-    def __init__(self, geom, offset_max, geom_true=None, exclusion_mask=None):
+    def __init__(
+        self,
+        geom,
+        offset_max,
+        geom_true=None,
+        exclusion_mask=None,
+        background_oversampling=None,
+    ):
         if not isinstance(geom, WcsGeom):
             raise ValueError("MapMaker only works with WcsGeom")
 
@@ -38,21 +51,12 @@ class MapMaker:
             raise ValueError("MapMaker only works with geom with an energy axis")
 
         self.geom = geom
-        self.geom_true = geom_true if geom_true else geom
+        self.geom_true = geom_true if geom_true else geom.to_binsz(BINSZ_IRF)
         self.offset_max = Angle(offset_max)
         self.exclusion_mask = exclusion_mask
+        self.background_oversampling = background_oversampling
 
-    def _get_empty_maps(self, selection):
-        # Initialise zero-filled maps
-        maps = {}
-        for name in selection:
-            if name == "exposure":
-                maps[name] = Map.from_geom(self.geom_true, unit="m2 s")
-            else:
-                maps[name] = Map.from_geom(self.geom, unit="")
-        return maps
-
-    def run(self, observations, selection=None):
+    def run(self, observations, selection=["counts", "exposure", "background"]):
         """Make maps for a list of observations.
 
         Parameters
@@ -69,26 +73,28 @@ class MapMaker:
         maps : dict
             Stacked counts, background and exposure maps
         """
+
         selection = _check_selection(selection)
-        maps = self._get_empty_maps(selection)
+
+        stacked = MapDataset.create(geom=self.geom, geom_irf=self.geom_true)
 
         for obs in observations:
-            log.info("Processing observation: OBS_ID = {}".format(obs.obs_id))
+            log.info(f"Processing observation: OBS_ID = {obs.obs_id}")
 
             try:
                 obs_maker = self._get_obs_maker(obs)
             except NoOverlapError:
-                log.info("Skipping observation {} (no map overlap)".format(obs.obs_id))
+                log.info(f"Skipping observation {obs.obs_id} (no map overlap)")
                 continue
 
-            maps_obs = obs_maker.run(selection)
+            dataset = obs_maker.run(selection)
+            stacked.stack(dataset)
 
-            for name in selection:
-                data = maps_obs[name].quantity.to_value(maps[name].unit)
-                if name == "exposure":
-                    maps[name].fill_by_coord(obs_maker.coords_etrue, data)
-                else:
-                    maps[name].fill_by_coord(obs_maker.coords, data)
+        maps = {
+            "counts": stacked.counts,
+            "exposure": stacked.exposure,
+            "background": stacked.background_model.evaluate(),
+        }
         self._maps = maps
         return maps
 
@@ -115,6 +121,7 @@ class MapMaker:
             geom_true=cutout_geom_etrue,
             offset_max=self.offset_max,
             exclusion_mask=cutout_exclusion,
+            background_oversampling=self.background_oversampling,
         )
 
     @staticmethod
@@ -123,7 +130,7 @@ class MapMaker:
 
         Parameters
         ----------
-        spectrum : `~gammapy.spectrum.models.SpectralModel`
+        spectrum : `~gammapy.modeling.models.SpectralModel`
             Spectral model to compute the weights.
             Default is power-law with spectral index of 2.
         keepdims : bool, optional
@@ -135,6 +142,7 @@ class MapMaker:
             if name == "exposure":
                 map = _map_spectrum_weight(map, spectrum)
             images[name] = map.sum_over_axes(keepdims=keepdims)
+        # TODO: PSF (and edisp) map sum_over_axis
 
         return images
 
@@ -154,7 +162,7 @@ class MapMaker:
         ----------
         observations : `~gammapy.data.Observations`
             Observations to process
-        spectrum : `~gammapy.spectrum.models.SpectralModel`
+        spectrum : `~gammapy.modeling.models.SpectralModel`
             Spectral model to compute the weights.
             Default is power-law with spectral index of 2.
         keepdims : bool, optional
@@ -181,35 +189,47 @@ class MapMakerObs:
     observation : `~gammapy.data.DataStoreObservation`
         Observation
     geom : `~gammapy.maps.WcsGeom`
-        Reference image geometry
+        Reference image geometry in reco energy, used for counts and background maps
     offset_max : `~astropy.coordinates.Angle`
         Maximum offset angle
     geom_true : `~gammapy.maps.WcsGeom`
-        Reference image geometry in true energy, used for exposure maps and PSF.
+        Reference image geometry in true energy, used for IRF maps. It can have a coarser
+        spatial bins than the counts geom.
         If none, the same as geom is assumed
     exclusion_mask : `~gammapy.maps.Map`
         Exclusion mask (used by some background estimators)
+    migra_axis : `~gammapy.maps.MapAxis`
+        Migration axis for edisp map
+    rad_axis : `~gammapy.maps.MapAxis`
+        Radial axis for psf map
     """
 
     def __init__(
-        self, observation, geom, offset_max, geom_true=None, exclusion_mask=None
+        self,
+        observation,
+        geom,
+        offset_max,
+        geom_true=None,
+        exclusion_mask=None,
+        background_oversampling=None,
+        migra_axis=None,
+        rad_axis=None,
     ):
         self.observation = observation
         self.geom = geom
-        self.geom_true = geom_true if geom_true else geom
+        self.geom_true = geom_true if geom_true else geom.to_binsz(BINSZ_IRF)
         self.offset_max = offset_max
         self.exclusion_mask = exclusion_mask
+        self.background_oversampling = background_oversampling
         self.maps = {}
+        self.migra_axis = migra_axis if migra_axis else MIGRA_AXIS_DEFAULT
+        self.rad_axis = rad_axis if rad_axis else RAD_AXIS_DEFAULT
 
     def _fov_mask(self, coords):
         pointing = self.observation.pointing_radec
         offset = coords.skycoord.separation(pointing)
         return offset >= self.offset_max
 
-    @lazyproperty
-    def fov_mask_etrue(self):
-        return self._fov_mask(self.coords_etrue)
-
     @lazyproperty
     def fov_mask(self):
         return self._fov_mask(self.coords)
@@ -220,13 +240,12 @@ class MapMakerObs:
 
     @lazyproperty
     def coords_etrue(self):
-        # Compute field of view mask on the cutout in true energy
         return self.geom_true.get_coord()
 
     def run(self, selection=None):
         """Make maps.
 
-        Returns dict with keys "counts", "exposure" and "background".
+        Returns dict with keys "counts", "exposure" and "background", "psf" and "edisp".
 
         Parameters
         ----------
@@ -240,7 +259,24 @@ class MapMakerObs:
         for name in selection:
             getattr(self, "_make_" + name)()
 
-        return self.maps
+        bkg = self.maps.get("background")
+
+        if bkg is not None:
+            background_model = BackgroundModel(bkg)
+        else:
+            background_model = None
+
+        dataset = MapDataset(
+            counts=self.maps.get("counts"),
+            exposure=self.maps.get("exposure"),
+            background_model=background_model,
+            psf=self.maps.get("psf"),
+            edisp=self.maps.get("edisp"),
+            gti=self.observation.gti,
+            name="obs_{}".format(self.observation.obs_id),
+            mask_safe=~self.fov_mask,
+        )
+        return dataset
 
     def _make_counts(self):
         counts = Map.from_geom(self.geom)
@@ -250,14 +286,31 @@ class MapMakerObs:
         self.maps["counts"] = counts
 
     def _make_exposure(self):
-        exposure = make_map_exposure_true_energy(
+        exposure_irf = make_map_exposure_true_energy(
             pointing=self.observation.pointing_radec,
             livetime=self.observation.observation_live_time_duration,
             aeff=self.observation.aeff,
             geom=self.geom_true,
         )
-        if self.fov_mask_etrue is not None:
-            exposure.data[..., self.fov_mask_etrue] = 0
+        mask_irf = self._fov_mask(self.geom_true.to_image().get_coord())
+        exposure_irf_masked = exposure_irf.copy()
+        exposure_irf_masked.data[..., mask_irf] = 0
+        # the exposure associated with the IRFS
+        self.maps["exposure_irf"] = exposure_irf_masked
+
+        energy_axis = self.geom_true.get_axis_by_name("energy")
+        geom = self.geom.to_image().to_cube([energy_axis])
+
+        exposure = make_map_exposure_true_energy(
+            pointing=self.observation.pointing_radec,
+            livetime=self.observation.observation_live_time_duration,
+            aeff=self.observation.aeff,
+            geom=geom,
+        )
+
+        fov_mask_etrue = self._fov_mask(geom.to_image().get_coord())
+        if fov_mask_etrue is not None:
+            exposure.data[..., fov_mask_etrue] = 0
         self.maps["exposure"] = exposure
 
     def _make_background(self):
@@ -268,14 +321,15 @@ class MapMakerObs:
             pnt = self.observation.pointing_radec
         else:
             raise ValueError(
-                'Found unknown background coordinate system definition: "{}". '
-                'Should be "ALTAZ" or "RADEC".'.format(bkg_coordsys)
+                f"Invalid background coordinate system: {bkg_coordsys!r}\n"
+                "Options: ALTAZ, RADEC"
             )
         background = make_map_background_irf(
             pointing=pnt,
             ontime=self.observation.observation_time_duration,
             bkg=self.observation.bkg,
             geom=self.geom,
+            oversampling=self.background_oversampling,
         )
         if self.fov_mask is not None:
             background.data[..., self.fov_mask] = 0
@@ -285,10 +339,38 @@ class MapMakerObs:
 
         self.maps["background"] = background
 
+    def _make_edisp(self):
+        energy_axis = self.geom_true.get_axis_by_name("ENERGY")
+        geom_migra = self.geom_true.to_image().to_cube([self.migra_axis, energy_axis])
+        edisp_map = make_edisp_map(
+            edisp=self.observation.edisp,
+            pointing=self.observation.pointing_radec,
+            geom=geom_migra,
+            max_offset=self.offset_max,
+            exposure_map=self.maps["exposure_irf"],
+        )
+        self.maps["edisp"] = edisp_map
+
+    def _make_psf(self):
+        psf = self.observation.psf
+        if isinstance(psf, EnergyDependentMultiGaussPSF):
+            psf = psf.to_psf3d()
+        energy_axis = self.geom_true.get_axis_by_name("ENERGY")
+
+        geom_rad = self.geom_true.to_image().to_cube([self.rad_axis, energy_axis])
+        psf_map = make_psf_map(
+            psf=psf,
+            pointing=self.observation.pointing_radec,
+            geom=geom_rad,
+            max_offset=self.offset_max,
+            exposure_map=self.maps["exposure_irf"],
+        )
+        self.maps["psf"] = psf_map
+
 
 def _check_selection(selection):
     """Handle default and validation of selection"""
-    available = ["counts", "exposure", "background"]
+    available = ["counts", "exposure", "background", "psf", "edisp"]
     if selection is None:
         selection = available
 
@@ -297,7 +379,7 @@ def _check_selection(selection):
 
     for name in selection:
         if name not in available:
-            raise ValueError("Selection not available: {!r}".format(name))
+            raise ValueError(f"Selection not available: {name!r}")
 
     return selection
 
@@ -307,7 +389,7 @@ class MapMakerRing(MapMaker):
 
     The main motivation for this class in addition to the `MapMaker`
     is to have the common image background estimation methods,
-    like `~gammapy.background.RingBackgroundEstimator`,
+    like `~gammapy.cube.RingBackgroundEstimator`,
     that work using on and off maps.
 
     To ensure adequate statistics, only observations that are fully
@@ -321,8 +403,8 @@ class MapMakerRing(MapMaker):
         Maximum offset angle
     exclusion_mask : `~gammapy.maps.Map`
         Exclusion mask
-    background_estimator : `~gammapy.background.RingBackgroundEstimator`
-        or `~gammapy.background.AdaptiveRingBackgroundEstimator`
+    background_estimator : `~gammapy.cube.RingBackgroundEstimator`
+        or `~gammapy.cube.AdaptiveRingBackgroundEstimator`
         Ring background estimator or something with an equivalent API.
 
     Examples
@@ -334,9 +416,8 @@ class MapMakerRing(MapMaker):
         from astropy.coordinates import SkyCoord
         from regions import CircleSkyRegion
         from gammapy.maps import Map, WcsGeom, MapAxis
-        from gammapy.cube import MapMakerRing
+        from gammapy.cube import MapMakerRing, RingBackgroundEstimator
         from gammapy.data import DataStore
-        from gammapy.background import RingBackgroundEstimator
 
         # Create observation list
         data_store = DataStore.from_file(
@@ -375,6 +456,16 @@ class MapMakerRing(MapMaker):
         )
         self.background_estimator = background_estimator
 
+    def _get_empty_maps(self, selection):
+        # Initialise zero-filled maps
+        maps = {}
+        for name in selection:
+            if name == "exposure":
+                maps[name] = Map.from_geom(self.geom, unit="m2 s")
+            else:
+                maps[name] = Map.from_geom(self.geom, unit="")
+        return maps
+
     def _run(self, observations, sum_over_axis=False, spectrum=None, keepdims=False):
         selection = ["on", "exposure_on", "off", "exposure_off", "exposure"]
         maps = self._get_empty_maps(selection)
@@ -383,15 +474,16 @@ class MapMakerRing(MapMaker):
 
         for obs in observations:
             try:
-                obs_maker = self._get_obs_maker(obs, mode="strict")
+                obs_maker = self._get_obs_maker(obs, mode="trim")
             except NoOverlapError:
-                log.info("Skipping obs_id: {} (no map overlap)".format(obs.obs_id))
-                continue
-            except PartialOverlapError:
-                log.info("Skipping obs_id: {} (partial map overlap)".format(obs.obs_id))
+                log.info(f"Skipping obs_id: {obs.obs_id} (no map overlap)")
                 continue
 
-            maps_obs = obs_maker.run()
+            dataset = obs_maker.run(selection=["counts", "exposure", "background"])
+            maps_obs = {}
+            maps_obs["counts"] = dataset.counts
+            maps_obs["exposure"] = dataset.exposure
+            maps_obs["background"] = dataset.background_model.map
             maps_obs["exclusion"] = obs_maker.exclusion_mask
 
             if sum_over_axis:
@@ -425,7 +517,7 @@ class MapMakerRing(MapMaker):
         ----------
         observations : `~gammapy.data.Observations`
             Observations to process
-        spectrum : `~gammapy.spectrum.models.SpectralModel`, optional
+        spectrum : `~gammapy.modeling.models.SpectralModel`, optional
             Spectral model to compute the weights.
             Default is power-law with spectral index of 2.
         keepdims : bool, optional
