@@ -1,11 +1,15 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Simulate observations"""
 import astropy.units as u
-from ..cube import MapDataset, PSFKernel
-from ..cube import make_map_exposure_true_energy, make_map_background_irf
-from ..maps import WcsNDMap
-from ..cube.models import BackgroundModel
-from ..utils.random import get_random_state
+from gammapy.cube import (
+    MapDataset,
+    PSFKernel,
+    make_map_background_irf,
+    make_map_exposure_true_energy,
+)
+from gammapy.maps import WcsNDMap
+from gammapy.modeling.models import BackgroundModel
+from gammapy.utils.random import get_random_state
 
 __all__ = ["simulate_dataset"]
 
@@ -21,7 +25,7 @@ def simulate_dataset(
     random_state="random-seed",
 ):
     """Simulate a 3D dataset.
-    
+
     Simulate a source defined with a sky model for a given pointing,
     geometry and irfs for a given exposure time.
     This will return a dataset object which includes the counts cube,
@@ -29,7 +33,7 @@ def simulate_dataset(
 
     Parameters
     ----------
-    skymodel : `~gammapy.cube.models.SkyModel`
+    skymodel : `~gammapy.modeling.models.SkyModel`
         Background model map
     geom : `~gammapy.maps.WcsGeom`
         Geometry object for the observation
