@@ -1,18 +1,17 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Functions to compute TS images."""
-import logging
 import contextlib
+import functools
+import logging
+import multiprocessing
 import warnings
-from functools import partial
-from multiprocessing import Pool
 import numpy as np
-from scipy.optimize import newton, brentq
+import scipy.optimize
 from astropy.convolution import CustomKernel, Kernel2D
-from ..utils.array import shape_2N, symmetric_crop_pad_width
+from gammapy.stats import cash, cash_sum_cython
+from gammapy.utils.array import shape_2N, symmetric_crop_pad_width
 from ._test_statistics_cython import (
-    _cash_cython,
     _amplitude_bounds_cython,
-    _cash_sum_cython,
     _f_cash_root_cython,
     _x_best_leastsq,
 )
@@ -65,12 +64,13 @@ def f_cash(x, counts, background, model):
     model : `~numpy.ndarray`
         Source template (multiplied with exposure).
     """
-    return _cash_sum_cython(counts, background + x * FLUX_FACTOR * model)
+    return cash_sum_cython(
+        counts.ravel(), (background + x * FLUX_FACTOR * model).ravel()
+    )
 
 
 class TSMapEstimator:
-    """
-    Compute TS map using different optimization methods.
+    r"""Compute TS map using different optimization methods.
 
 
     The map is computed fitting by a single parameter amplitude fit. The fit is
@@ -114,13 +114,12 @@ class TSMapEstimator:
     Negative :math:`TS` values are defined as following:
 
     .. math::
-
-        TS = \\left \\{
-                 \\begin{array}{ll}
-                   -TS & : \\textnormal{if} \\ F < 0 \\\\
-                   \\ \\ TS & : \\textnormal{else}
-                 \\end{array}
-               \\right.
+        TS = \left \{
+                 \begin{array}{ll}
+                   -TS \text{ if } F < 0 \\
+                    TS \text{ else}
+                 \end{array}
+               \right.
 
     Where :math:`F` is the fitted flux amplitude.
 
@@ -142,10 +141,10 @@ class TSMapEstimator:
     ):
 
         if method not in ["root brentq", "root newton", "leastsq iter"]:
-            raise ValueError("Not a valid method: '{}'".format(method))
+            raise ValueError(f"Not a valid method: '{method}'")
 
         if error_method not in ["covar", "conf"]:
-            raise ValueError("Not a valid error method '{}'".format(error_method))
+            raise ValueError(f"Not a valid error method '{error_method}'")
 
         self.parameters = {
             "method": method,
@@ -220,8 +219,8 @@ class TSMapEstimator:
         .. math::
             \sqrt{TS} = \left \{
             \begin{array}{ll}
-              -\sqrt{-TS} & : \textnormal{if} \ TS < 0 \\
-              \sqrt{TS} & : \textnormal{else}
+              -\sqrt{-TS} & : \text{if} \ TS < 0 \\
+              \sqrt{TS} & : \text{else}
             \end{array}
             \right.
 
@@ -273,15 +272,17 @@ class TSMapEstimator:
             )
 
         if downsampling_factor:
+            maps_downsampled = {}
+
             shape = maps["counts"].data.shape
             pad_width = symmetric_crop_pad_width(shape, shape_2N(shape))[0]
 
-            for name in maps:
-                maps[name] = maps[name].pad(pad_width)
+            for name, map_ in maps.items():
                 preserve_counts = name in ["counts", "background", "exclusion"]
-                maps[name] = maps[name].downsample(
+                maps_downsampled[name] = map_.pad(pad_width).downsample(
                     downsampling_factor, preserve_counts=preserve_counts
                 )
+            maps = maps_downsampled
 
         if not isinstance(kernel, Kernel2D):
             kernel = CustomKernel(kernel)
@@ -310,12 +311,12 @@ class TSMapEstimator:
         exposure = maps["exposure"].data.astype(float)
 
         # Compute null statistics per pixel for the whole image
-        c_0 = _cash_cython(counts, background)
+        c_0 = cash(counts, background)
 
         error_method = p["error_method"] if "flux_err" in which else "none"
         ul_method = p["ul_method"] if "flux_ul" in which else "none"
 
-        wrap = partial(
+        wrap = functools.partial(
             _ts_value,
             counts=counts,
             exposure=exposure,
@@ -335,8 +336,8 @@ class TSMapEstimator:
         x, y = np.where(mask.data)
         positions = list(zip(x, y))
 
-        with contextlib.closing(Pool(processes=p["n_jobs"])) as pool:
-            log.info("Using {} jobs to compute TS map.".format(p["n_jobs"]))
+        with contextlib.closing(multiprocessing.Pool(processes=p["n_jobs"])) as pool:
+            log.info(f"Number of jobs to compute TS map: {p['n_jobs']}")
             results = pool.map(wrap, positions)
 
         pool.join()
@@ -371,7 +372,7 @@ class TSMapEstimator:
         info = self.__class__.__name__
         info += "\n\nParameters:\n\n"
         for key in p:
-            info += "\t{key:13s}: {value}\n".format(key=key, value=p[key])
+            info += f"\t{key:13s}: {p[key]}\n"
         return info
 
 
@@ -453,7 +454,7 @@ def _ts_value(
             counts_, background_, model, rtol=rtol
         )
     else:
-        raise ValueError("Invalid method: {}".format(method))
+        raise ValueError(f"Invalid method: {method}")
 
     with np.errstate(invalid="ignore", divide="ignore"):
         c_1 = f_cash(amplitude, counts_, background_, model)
@@ -552,7 +553,7 @@ def _root_amplitude(counts, background, model, flux, rtol=RTOL):
         warnings.simplefilter("ignore")
         try:
             return (
-                newton(
+                scipy.optimize.newton(
                     _f_cash_root_cython, flux, args=args, maxiter=MAX_NITER, tol=rtol
                 ),
                 0,
@@ -594,7 +595,7 @@ def _root_amplitude_brentq(counts, background, model, rtol=RTOL):
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
         try:
-            result = brentq(
+            result = scipy.optimize.brentq(
                 _f_cash_root_cython,
                 amplitude_min,
                 amplitude_max,
@@ -632,7 +633,7 @@ def _compute_flux_err_conf(amplitude, counts, background, model, c_1, error_sigm
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
         try:
-            result = brentq(
+            result = scipy.optimize.brentq(
                 ts_diff,
                 amplitude,
                 amplitude_max,
