@@ -1,8 +1,10 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import logging
 import numpy as np
-from ..utils.energy import Energy
-from . import EnergyDependentTablePSF, IRFStacker, EffectiveAreaTable
+from astropy import units as u
+from .effective_area import EffectiveAreaTable
+from .irf_stack import IRFStacker
+from .psf_table import EnergyDependentTablePSF
 
 __all__ = [
     "make_psf",
@@ -94,8 +96,8 @@ def make_mean_edisp(
     position,
     e_true,
     e_reco,
-    low_reco_threshold=Energy(0.002, "TeV"),
-    high_reco_threshold=Energy(150, "TeV"),
+    low_reco_threshold="0.002 TeV",
+    high_reco_threshold="150 TeV",
 ):
     """Compute mean energy dispersion.
 
@@ -109,20 +111,23 @@ def make_mean_edisp(
         Observations for which to compute the EDISP
     position : `~astropy.coordinates.SkyCoord`
         Position at which to compute the EDISP
-    e_true : `~gammapy.utils.energy.EnergyBounds`
+    e_true : `~astropy.units.Quantity`
         True energy axis
-    e_reco : `~gammapy.utils.energy.EnergyBounds`
+    e_reco : `~astropy.units.Quantity`
         Reconstructed energy axis
-    low_reco_threshold : `~gammapy.utils.energy.Energy`
-        low energy threshold in reco energy, default 0.002 TeV
-    high_reco_threshold : `~gammapy.utils.energy.Energy`
-        high energy threshold in reco energy , default 150 TeV
+    low_reco_threshold : `~astropy.units.Quantity`
+        low energy threshold in reco energy
+    high_reco_threshold : `~astropy.units.Quantity`
+        high energy threshold in reco energy
 
     Returns
     -------
     stacked_edisp : `~gammapy.irf.EnergyDispersion`
         Stacked EDISP for a set of observation
     """
+    low_reco_threshold = u.Quantity(low_reco_threshold)
+    high_reco_threshold = u.Quantity(high_reco_threshold)
+
     list_aeff = []
     list_edisp = []
     list_livetime = []
@@ -234,7 +239,7 @@ def compute_energy_thresholds(
     elif method_lo == "none":
         thres_lo = aeff.energy.edges[0]
     else:
-        raise ValueError("Invalid method_lo: {}".format(method_lo))
+        raise ValueError(f"Invalid method_lo: {method_lo}")
 
     # High threshold
     if method_hi == "area_max":
@@ -255,6 +260,6 @@ def compute_energy_thresholds(
     elif method_hi == "none":
         thres_hi = aeff.energy.edges[-1]
     else:
-        raise ValueError("Invalid method_hi: {}".format(method_hi))
+        raise ValueError(f"Invalid method_hi: {method_hi}")
 
     return thres_lo, thres_hi
