@@ -1,16 +1,16 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import logging
 import numpy as np
-from astropy.io import fits
+import scipy.integrate
 from astropy import units as u
 from astropy.coordinates import Angle
+from astropy.io import fits
 from astropy.utils import lazyproperty
-from scipy.integrate import cumtrapz
-from ..utils.interpolation import ScaledRegularGridInterpolator
-from ..utils.gauss import Gauss2DPDF
-from ..utils.scripts import make_path
-from ..utils.array import array_stats_str
-from ..utils.energy import energy_logspace
+from gammapy.utils.array import array_stats_str
+from gammapy.utils.energy import energy_logspace
+from gammapy.utils.gauss import Gauss2DPDF
+from gammapy.utils.interpolation import ScaledRegularGridInterpolator
+from gammapy.utils.scripts import make_path
 
 __all__ = ["TablePSF", "EnergyDependentTablePSF"]
 
@@ -51,7 +51,9 @@ class TablePSF:
             rad = self.rad
 
         rad_drad = 2 * np.pi * rad * self.evaluate(rad)
-        values = cumtrapz(rad_drad.to_value("rad-1"), rad.to_value("rad"), initial=0)
+        values = scipy.integrate.cumtrapz(
+            rad_drad.to_value("rad-1"), rad.to_value("rad"), initial=0
+        )
 
         return ScaledRegularGridInterpolator(points=(rad,), values=values, fill_value=1)
 
@@ -93,7 +95,7 @@ class TablePSF:
             gauss2d_pdf = Gauss2DPDF(sigma=width.radian)
             psf_value = gauss2d_pdf(rad.radian)
         else:
-            raise ValueError("Invalid shape: {}".format(shape))
+            raise ValueError(f"Invalid shape: {shape}")
 
         psf_value = u.Quantity(psf_value, "sr^-1")
 
@@ -102,11 +104,11 @@ class TablePSF:
     def info(self):
         """Print basic info."""
         ss = array_stats_str(self.rad.deg, "offset")
-        ss += "integral = {}\n".format(self.integral())
+        ss += f"integral = {self.integral()}\n"
 
         for containment in [68, 80, 95]:
             radius = self.containment_radius(0.01 * containment)
-            ss += "containment radius {} deg for {}%\n".format(radius.deg, containment)
+            ss += f"containment radius {radius.deg} deg for {containment}%\n"
 
         return ss
 
@@ -277,7 +279,7 @@ class EnergyDependentTablePSF:
             rad = self.rad
 
         rad_drad = 2 * np.pi * rad * self.evaluate(energy=self.energy, rad=rad)
-        values = cumtrapz(
+        values = scipy.integrate.cumtrapz(
             rad_drad.to_value("rad-1"), rad.to_value("rad"), initial=0, axis=1
         )
 
@@ -297,9 +299,8 @@ class EnergyDependentTablePSF:
         for fraction in fractions:
             rads = self.containment_radius(energy=energies, fraction=fraction)
             for energy, rad in zip(energies, rads):
-                ss += "  " + "{}% containment radius at {:3.0f}: {:.2f}\n".format(
-                    100 * fraction, energy, rad
-                )
+                ss += f"  {100 * fraction}% containment radius at {energy:3.0f}: {rad:.2f}\n"
+
         return ss
 
     @classmethod
@@ -414,7 +415,7 @@ class EnergyDependentTablePSF:
         ----------
         energy_band : `~astropy.units.Quantity`
             Energy band
-        spectrum : `SpectralModel`
+        spectrum : `~gammapy.modeling.models.SpectralModel`
             Spectral model used for weighting the PSF. Default is a power law
             with index=2.
         n_bins : int
@@ -426,12 +427,12 @@ class EnergyDependentTablePSF:
         psf : `TablePSF`
             Table PSF
         """
-        from ..spectrum.models import PowerLaw, TableModel
+        from gammapy.modeling.models import PowerLawSpectralModel, TemplateSpectralModel
 
         if spectrum is None:
-            spectrum = PowerLaw()
+            spectrum = PowerLawSpectralModel()
 
-        exposure = TableModel(self.energy, self.exposure)
+        exposure = TemplateSpectralModel(self.energy, self.exposure)
 
         e_min, e_max = energy_band
         energy = energy_logspace(emin=e_min, emax=e_max, nbins=n_bins)
@@ -514,12 +515,12 @@ class EnergyDependentTablePSF:
 
         for energy in energies:
             psf_value = np.squeeze(self.evaluate(energy=energy))
-            label = "{:.0f}".format(energy)
+            label = f"{energy:.0f}"
             ax.plot(
                 self.rad.to_value("deg"),
                 psf_value.to_value("sr-1"),
                 label=label,
-                **kwargs
+                **kwargs,
             )
 
         ax.set_yscale("log")
@@ -538,7 +539,7 @@ class EnergyDependentTablePSF:
 
         for fraction in fractions:
             rad = self.containment_radius(self.energy, fraction)
-            label = "{:.1f}% Containment".format(100 * fraction)
+            label = f"{100 * fraction:.1f}% Containment"
             ax.plot(self.energy.value, rad.value, label=label, **kwargs)
 
         ax.semilogx()
