@@ -3,23 +3,17 @@ import abc
 import copy
 import inspect
 import json
-from collections import OrderedDict
 import numpy as np
 from astropy import units as u
-from astropy.utils.misc import InheritDocstrings
 from astropy.io import fits
-from .geom import pix_tuple_to_idx, MapCoord
-from .utils import unpack_seq, INVALID_VALUE
-from ..utils.scripts import make_path
+from gammapy.utils.scripts import make_path
+from .geom import MapCoord, pix_tuple_to_idx
+from .utils import INVALID_VALUE
 
 __all__ = ["Map"]
 
 
-class MapMeta(InheritDocstrings, abc.ABCMeta):
-    pass
-
-
-class Map(metaclass=MapMeta):
+class Map(abc.ABC):
     """Abstract map class.
 
     This can represent WCS- or HEALPIX-based maps
@@ -27,12 +21,12 @@ class Map(metaclass=MapMeta):
 
     Parameters
     ----------
-    geom : `~gammapy.maps.MapGeom`
+    geom : `~gammapy.maps.Geom`
         Geometry
     data : `~numpy.ndarray`
         Data array
-    meta : `~collections.OrderedDict`
-        Dictionary to store meta data.
+    meta : `dict`
+        Dictionary to store meta data
     unit : str or `~astropy.units.Unit`
         Data unit
     """
@@ -62,7 +56,7 @@ class Map(metaclass=MapMeta):
 
     @property
     def geom(self):
-        """Map geometry (`~gammapy.maps.MapGeom`)"""
+        """Map geometry (`~gammapy.maps.Geom`)"""
         return self._geom
 
     @geom.setter
@@ -78,8 +72,7 @@ class Map(metaclass=MapMeta):
     def data(self, val):
         if val.shape != self.geom.data_shape:
             raise ValueError(
-                "Shape {!r} does not match map data shape {!r}"
-                "".format(val.shape, self.geom.data_shape)
+                f"Shape {val.shape!r} does not match map data shape {self.geom.data_shape!r}"
             )
 
         if isinstance(val, u.Quantity):
@@ -98,12 +91,12 @@ class Map(metaclass=MapMeta):
 
     @property
     def meta(self):
-        """Map meta (`~collections.OrderedDict`)"""
+        """Map meta (`dict`)"""
         return self._meta
 
     @meta.setter
     def meta(self, val):
-        self._meta = OrderedDict(val)
+        self._meta = val
 
     @property
     def quantity(self):
@@ -143,7 +136,7 @@ class Map(metaclass=MapMeta):
             Data type, default is 'float32'
         unit : str or `~astropy.units.Unit`
             Data unit.
-        meta : `~collections.OrderedDict`
+        meta : `dict`
             Dictionary to store meta data.
 
         Returns
@@ -160,7 +153,7 @@ class Map(metaclass=MapMeta):
         elif "hpx" in map_type.lower():
             return HpxMap.create(**kwargs)
         else:
-            raise ValueError("Unrecognized map type: {!r}".format(map_type))
+            raise ValueError(f"Unrecognized map type: {map_type!r}")
 
     @staticmethod
     def read(filename, hdu=None, hdu_bands=None, map_type="auto"):
@@ -193,16 +186,18 @@ class Map(metaclass=MapMeta):
             return Map.from_hdulist(hdulist, hdu, hdu_bands, map_type)
 
     @staticmethod
-    def from_geom(geom, meta=None, data=None, map_type="auto", unit=""):
-        """Generate an empty map from a `MapGeom` instance.
+    def from_geom(
+        geom, meta=None, data=None, map_type="auto", unit="", dtype="float32"
+    ):
+        """Generate an empty map from a `Geom` instance.
 
         Parameters
         ----------
-        geom : `MapGeom`
+        geom : `Geom`
             Map geometry.
         data : `numpy.ndarray`
             data array
-        meta : `~collections.OrderedDict`
+        meta : `dict`
             Dictionary to store meta data.
         map_type : {'wcs', 'wcs-sparse', 'hpx', 'hpx-sparse', 'auto'}
             Map type.  Selects the class that will be used to
@@ -231,7 +226,7 @@ class Map(metaclass=MapMeta):
                 raise ValueError("Unrecognized geom type.")
 
         cls_out = Map._get_map_cls(map_type)
-        return cls_out(geom, data=data, meta=meta, unit=unit)
+        return cls_out(geom, data=data, meta=meta, unit=unit, dtype=dtype)
 
     @staticmethod
     def from_hdulist(hdulist, hdu=None, hdu_bands=None, map_type="auto"):
@@ -245,10 +240,9 @@ class Map(metaclass=MapMeta):
     def _get_meta_from_header(header):
         """Load meta data from a FITS header."""
         if "META" in header:
-            meta = json.loads(header["META"], object_pairs_hook=OrderedDict)
+            return json.loads(header["META"])
         else:
-            meta = OrderedDict()
-        return meta
+            return {}
 
     @staticmethod
     def _get_map_type(hdu_list, hdu_name):
@@ -293,7 +287,7 @@ class Map(metaclass=MapMeta):
 
             return HpxSparseMap
         else:
-            raise ValueError("Unrecognized map type: {!r}".format(map_type))
+            raise ValueError(f"Unrecognized map type: {map_type!r}")
 
     def write(self, filename, overwrite=False, **kwargs):
         """Write to a FITS file.
@@ -328,90 +322,43 @@ class Map(metaclass=MapMeta):
         hdulist.writeto(filename, overwrite=overwrite)
 
     def iter_by_image(self):
-        """Iterate over image planes of the map returning a tuple with the image
-        array and image plane index.
+        """Iterate over image planes of the map.
+
+        This is a generator yielding ``(data, idx)`` tuples,
+        where ``data`` is a `numpy.ndarray` view of the image plane data,
+        and ``idx`` is a tuple of int, the index of the image plane.
 
         The image plane index is in data order, so that the data array can be
         indexed directly. See :ref:`mapiter` for further information.
-
-        Returns
-        -------
-        val : `~numpy.ndarray`
-            Array of image plane values.
-        idx : tuple
-            Index of image plane.
         """
         for idx in np.ndindex(self.geom.shape_axes):
             yield self.data[idx[::-1]], idx[::-1]
 
-    def iter_by_pix(self, buffersize=1):
-        """Iterate over elements of the map returning a tuple with values and
-        pixel coordinates.
-
-        Parameters
-        ----------
-        buffersize : int
-            Set the size of the buffer.  The map will be returned in
-            chunks of the given size.
-
-        Returns
-        -------
-        val : `~numpy.ndarray`
-            Map values.
-        pix : tuple
-            Tuple of pixel coordinates.
-        """
-        pix = list(self.geom.get_idx(flat=True))
-        vals = self.data[np.isfinite(self.data)]
-        x = [vals] + pix
-        return unpack_seq(
-            np.nditer(x, flags=["external_loop", "buffered"], buffersize=buffersize)
-        )
-
-    def iter_by_coord(self, buffersize=1):
-        """Iterate over elements of the map returning a tuple with values and
-        map coordinates.
-
-        Parameters
-        ----------
-        buffersize : int
-            Set the size of the buffer.  The map will be returned in
-            chunks of the given size.
-
-        Returns
-        -------
-        val : `~numpy.ndarray`
-            Map values.
-        coords : tuple
-            Tuple of map coordinates.
-        """
-        coords = list(self.geom.get_coord(flat=True))
-        vals = self.data[np.isfinite(self.data)]
-        x = [vals] + coords
-        return unpack_seq(
-            np.nditer(x, flags=["external_loop", "buffered"], buffersize=buffersize)
-        )
-
     @abc.abstractmethod
     def sum_over_axes(self, keepdims=False):
         """Reduce to a 2D image by summing over non-spatial dimensions."""
         pass
 
-    def coadd(self, map_in):
-        """Add the contents of ``map_in`` to this map.  This method can be
-        used to combine maps containing integral quantities (e.g. counts)
+    def coadd(self, map_in, weights=None):
+        """Add the contents of ``map_in`` to this map.
+
+        This method can be used to combine maps containing integral quantities (e.g. counts)
         or differential quantities if the maps have the same binning.
 
         Parameters
         ----------
         map_in : `Map`
             Input map.
+        weights: `Map` or `~numpy.ndarray`
+            The weight factors while adding
         """
         if not self.unit.is_equivalent(map_in.unit):
             raise ValueError("Incompatible units")
 
         # TODO: Check whether geometries are aligned and if so sum the
         # data vectors directly
+        if weights is not None:
+            map_in = map_in * weights
         idx = map_in.geom.get_idx()
         coords = map_in.geom.get_coord()
         vals = u.Quantity(map_in.get_by_idx(idx), map_in.unit)
@@ -425,7 +372,7 @@ class Map(metaclass=MapMeta):
 
         Parameters
         ----------
-        geom : `MapGeom`
+        geom : `Geom`
             Geometry of projection.
         mode : {'interp', 'exact'}
             Method for reprojection.  'interp' method interpolates at pixel
@@ -461,8 +408,7 @@ class Map(metaclass=MapMeta):
 
     @abc.abstractmethod
     def pad(self, pad_width, mode="constant", cval=0, order=1):
-        """Pad the spatial dimension of the map by extending the edge of the
-        map by the given number of pixels.
+        """Pad the spatial dimensions of the map.
 
         Parameters
         ----------
@@ -488,8 +434,7 @@ class Map(metaclass=MapMeta):
 
     @abc.abstractmethod
     def crop(self, crop_width):
-        """Crop the spatial dimension of the map by removing a number of
-        pixels from the edge of the map.
+        """Crop the spatial dimensions of the map.
 
         Parameters
         ----------
@@ -505,7 +450,7 @@ class Map(metaclass=MapMeta):
         pass
 
     @abc.abstractmethod
-    def downsample(self, factor, preserve_counts=True):
+    def downsample(self, factor, preserve_counts=True, axis=None):
         """Downsample the spatial dimension by a given factor.
 
         Parameters
@@ -516,6 +461,8 @@ class Map(metaclass=MapMeta):
             Preserve the integral over each bin.  This should be true
             if the map is an integral quantity (e.g. counts) and false if
             the map is a differential quantity (e.g. intensity).
+        axis : str
+            Which axis to downsample. By default spatial axes are downsampled.
 
         Returns
         -------
@@ -525,7 +472,7 @@ class Map(metaclass=MapMeta):
         pass
 
     @abc.abstractmethod
-    def upsample(self, factor, order=0, preserve_counts=True):
+    def upsample(self, factor, order=0, preserve_counts=True, axis=None):
         """Upsample the spatial dimension by a given factor.
 
         Parameters
@@ -538,6 +485,9 @@ class Map(metaclass=MapMeta):
             Preserve the integral over each bin.  This should be true
             if the map is an integral quantity (e.g. counts) and false if
             the map is a differential quantity (e.g. intensity).
+        axis : str
+            Which axis to upsample. By default spatial axes are upsampled.
+
 
         Returns
         -------
@@ -629,7 +579,7 @@ class Map(metaclass=MapMeta):
         """
         if isinstance(coords, tuple):
             axes_names = [_.name for _ in self.geom.axes]
-            coords = OrderedDict(zip(axes_names, coords))
+            coords = dict(zip(axes_names, coords))
 
         idx = []
         for ax in self.geom.axes:
@@ -961,15 +911,15 @@ class Map(metaclass=MapMeta):
         for axis in self.geom.axes:
             if axis.node_type == "edges":
                 options = [
-                    "{:.2e} - {:.2e} {}".format(val_min, val_max, axis.unit)
+                    f"{val_min:.2e} - {val_max:.2e} {axis.unit}"
                     for val_min, val_max in zip(axis.edges[:-1], axis.edges[1:])
                 ]
             else:
-                options = ["{:.2e} {}".format(val, axis.unit) for val in axis.center]
+                options = [f"{val:.2e} {axis.unit}" for val in axis.center]
 
             interact_kwargs[axis.name] = SelectionSlider(
                 options=options,
-                description="Select {}:".format(axis.name),
+                description=f"Select {axis.name}:",
                 continuous_update=False,
                 style={"description_width": "initial"},
                 layout={"width": "50%"},
@@ -1013,18 +963,19 @@ class Map(metaclass=MapMeta):
         return self._init_copy(**kwargs)
 
     def __repr__(self):
-        str_ = self.__class__.__name__
-        str_ += "\n\n"
         geom = self.geom.__class__.__name__
-        str_ += "\tgeom  : {} \n ".format(geom)
         axes = ["skycoord"] if self.geom.is_hpx else ["lon", "lat"]
         axes = axes + [_.name for _ in self.geom.axes]
-        str_ += "\taxes  : {}\n".format(", ".join(axes))
-        str_ += "\tshape : {}\n".format(self.geom.data_shape[::-1])
-        str_ += "\tndim  : {}\n".format(self.geom.ndim)
-        str_ += "\tunit  : {!r} \n".format(str(self.unit))
-        str_ += "\tdtype : {} \n".format(self.data.dtype)
-        return str_
+
+        return (
+            f"{self.__class__.__name__}\n\n"
+            f"\tgeom  : {geom} \n "
+            f"\taxes  : {axes}\n"
+            f"\tshape : {self.geom.data_shape[::-1]}\n"
+            f"\tndim  : {self.geom.ndim}\n"
+            f"\tunit  : {self.unit}\n"
+            f"\tdtype : {self.data.dtype}\n"
+        )
 
     def _arithmetics(self, operator, other, copy):
         """Perform arithmetics on maps after checking geometry consistency."""
@@ -1063,3 +1014,6 @@ class Map(metaclass=MapMeta):
 
     def __itruediv__(self, other):
         return self._arithmetics(np.true_divide, other, copy=False)
+
+    def __array__(self):
+        return self.data
