@@ -1,21 +1,19 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from collections import OrderedDict
-import numpy as np
 from pathlib import Path
+import numpy as np
 from astropy import units as u
-from astropy.table import Table
 from astropy.io import fits
-from .utils import SpectrumEvaluator
-from ..utils.scripts import make_path
-from ..utils.fitting import Dataset, Parameters
-from ..utils.fits import energy_axis_to_ebounds
-from ..stats import wstat, cash
-from ..utils.random import get_random_state
-from ..data import ObservationStats
-from .core import CountsSpectrum
-from ..irf import EffectiveAreaTable, EnergyDispersion, IRFStacker
+from astropy.table import Table
+from gammapy.data import GTI, ObservationStats
+from gammapy.irf import EffectiveAreaTable, EnergyDispersion, IRFStacker
+from gammapy.modeling import Dataset, Parameters
+from gammapy.stats import cash, wstat
+from gammapy.utils.fits import energy_axis_to_ebounds
+from gammapy.utils.random import get_random_state
+from gammapy.utils.scripts import make_path
+from .core import CountsSpectrum, SpectrumEvaluator
 
-__all__ = ["SpectrumDatasetOnOff", "SpectrumDataset", "SpectrumDatasetOnOffStacker"]
+__all__ = ["SpectrumDatasetOnOff", "SpectrumDataset"]
 
 
 class SpectrumDataset(Dataset):
@@ -27,7 +25,7 @@ class SpectrumDataset(Dataset):
 
     Parameters
     ----------
-    model : `~gammapy.spectrum.models.SpectralModel`
+    model : `~gammapy.modeling.models.SpectralModel`
         Fit model
     counts : `~gammapy.spectrum.CountsSpectrum`
         Counts spectrum
@@ -43,15 +41,14 @@ class SpectrumDataset(Dataset):
         Mask defining the safe data range.
     mask_fit : `~numpy.ndarray`
         Mask to apply to the likelihood for fitting.
-    obs_id : int or list of int
-        Observation id(s) corresponding to the (stacked) dataset.
-    gti : '~gammapy.data.gti.GTI'
+    name : str
+        Dataset name.
+    gti : `~gammapy.data.GTI`
         GTI of the observation or union of GTI if it is a stacked observation
 
     See Also
     --------
-    SpectrumDatasetOnOff, FluxPointsDataset, MapDataset
-
+    SpectrumDatasetOnOff, FluxPointsDataset, gammapy.cube.MapDataset
     """
 
     likelihood_type = "cash"
@@ -66,7 +63,7 @@ class SpectrumDataset(Dataset):
         background=None,
         mask_safe=None,
         mask_fit=None,
-        obs_id=None,
+        name="",
         gti=None,
     ):
         if mask_fit is not None and mask_fit.dtype != np.dtype("bool"):
@@ -84,16 +81,15 @@ class SpectrumDataset(Dataset):
         self.background = background
         self.model = model
         self.mask_safe = mask_safe
-        self.obs_id = obs_id
+        self.name = name
         self.gti = gti
 
-    def __repr__(self):
-        str_ = self.__class__.__name__
-        return str_
-
     def __str__(self):
         str_ = self.__class__.__name__
         str_ += "\n\n"
+
+        str_ += "\t{:32}: {} \n\n".format("Name", self.name)
+
         counts = np.nan
         if self.counts is not None:
             counts = np.sum(self.counts.data)
@@ -181,6 +177,17 @@ class SpectrumDataset(Dataset):
             self._parameters = None
             self._predictor = None
 
+    @property
+    def mask_safe(self):
+        if self._mask_safe is None:
+            return np.ones(self.data_shape, bool)
+        else:
+            return self._mask_safe
+
+    @mask_safe.setter
+    def mask_safe(self, mask):
+        self._mask_safe = mask
+
     @property
     def parameters(self):
         if self._parameters is None:
@@ -205,7 +212,7 @@ class SpectrumDataset(Dataset):
         return (self._energy_axis.nbin,)
 
     def npred(self):
-        """Returns npred map (model + background)"""
+        """Return npred map (model + background)"""
         if self._predictor is None:
             raise AttributeError("No model set for Dataset")
         npred = self._predictor.compute_npred()
@@ -218,7 +225,7 @@ class SpectrumDataset(Dataset):
         return cash(n_on=self.counts.data, mu_on=self.npred().data)
 
     def _as_counts_spectrum(self, data):
-        energy = self.counts.energy.edges
+        energy = self._energy_axis.edges
         return CountsSpectrum(data=data, energy_lo=energy[:-1], energy_hi=energy[1:])
 
     @property
@@ -235,8 +242,8 @@ class SpectrumDataset(Dataset):
         Parameters
         ----------
         random_state : {int, 'random-seed', 'global-rng', `~numpy.random.RandomState`}
-                Defines random number generator initialisation.
-                Passed to `~gammapy.utils.random.get_random_state`.
+            Defines random number generator initialisation.
+            Passed to `~gammapy.utils.random.get_random_state`.
         """
         random_state = get_random_state(random_state)
         npred = self.npred()
@@ -246,7 +253,7 @@ class SpectrumDataset(Dataset):
     @property
     def energy_range(self):
         """Energy range defined by the safe mask"""
-        energy = self.counts.energy.edges
+        energy = self._energy_axis.edges
         e_lo = energy[:-1][self.mask_safe]
         e_hi = energy[1:][self.mask_safe]
         return u.Quantity([e_lo.min(), e_hi.max()])
@@ -308,18 +315,17 @@ class SpectrumDataset(Dataset):
 
         Parameters
         ----------
-        method: {"diff", "diff/model", "diff/sqrt(model)"}
+        method : {"diff", "diff/model", "diff/sqrt(model)"}
             Method used to compute the residuals. Available options are:
-                - `diff` (default): data - model
-                - `diff/model`: (data - model) / model
-                - `diff/sqrt(model)`: (data - model) / sqrt(model)
+                - ``diff`` (default): data - model
+                - ``diff/model``: (data - model) / model
+                - ``diff/sqrt(model)``: (data - model) / sqrt(model)
 
         Returns
         -------
         residuals : `CountsSpectrum`
-            Residual spectrum.
+            Residual spectrum
         """
-
         residuals = self._compute_residuals(self.counts, self.npred(), method)
         return residuals
 
@@ -355,10 +361,128 @@ class SpectrumDataset(Dataset):
         ymax = 1.2 * np.nanmax(residuals.data)
         ax.set_ylim(-ymax, ymax)
 
-        ax.set_xlabel("Energy [{}]".format(self._e_unit))
-        ax.set_ylabel("Residuals ({})".format(label))
+        ax.set_xlabel(f"Energy [{self._e_unit}]")
+        ax.set_ylabel(f"Residuals ({label})")
         return ax
 
+    @classmethod
+    def create(cls, e_reco, e_true=None, reference_time="2000-01-01"):
+        """Creates empty SpectrumDataset
+
+        Empty containers are created with the correct geometry.
+        counts, background and aeff are zero and edisp is diagonal.
+
+        The safe_mask is set to False in every bin.
+
+        Parameters
+        ----------
+        e_reco : `~astropy.units.Quantity`
+            edges of counts vector
+        e_true : `~astropy.units.Quantity`
+            edges of effective area table. If not set use reco energy values. Default : None
+        reference_time : `~astropy.time.Time`
+            reference time of the dataset, Default is "2000-01-01"
+        """
+        if e_true is None:
+            e_true = e_reco
+
+        counts = CountsSpectrum(e_reco[:-1], e_reco[1:])
+        background = CountsSpectrum(e_reco[:-1], e_reco[1:])
+        aeff = EffectiveAreaTable(
+            e_true[:-1], e_true[1:], np.zeros(e_true[:-1].shape) * u.m ** 2
+        )
+        edisp = EnergyDispersion.from_diagonal_response(e_true, e_reco)
+        mask_safe = np.ones_like(counts.data, "bool")
+        gti = GTI.create(u.Quantity([], "s"), u.Quantity([], "s"), reference_time)
+        livetime = gti.time_sum
+
+        return SpectrumDataset(
+            counts=counts,
+            aeff=aeff,
+            edisp=edisp,
+            mask_safe=mask_safe,
+            background=background,
+            livetime=livetime,
+            gti=gti,
+        )
+
+    def stack(self, other):
+        r"""Stack this dataset with another one.
+
+        Safe mask is applied to compute the stacked counts vector.
+        Counts outside each dataset safe mask are lost.
+
+        Stacking is performed in-place.
+
+        The stacking of 2 datasets is implemented as follows.
+        Here, :math:`k` denotes a bin in reconstructed energy and :math:`j = {1,2}` is the dataset number
+
+        The ``mask_safe`` of each dataset is defined as:
+
+        .. math::
+            \epsilon_{jk} =\left\{\begin{array}{cl} 1, &
+            \mbox{if bin k is inside the energy thresholds}\\ 0, &
+            \mbox{otherwise} \end{array}\right.
+
+        Then the total ``counts`` and model background ``bkg`` are computed according to:
+
+        .. math::
+            \overline{\mathrm{n_{on}}}_k =  \mathrm{n_{on}}_{1k} \cdot \epsilon_{1k} +
+             \mathrm{n_{on}}_{2k} \cdot \epsilon_{2k}
+
+            \overline{bkg}_k = bkg_{1k} \cdot \epsilon_{1k} +
+             bkg_{2k} \cdot \epsilon_{2k}
+
+        The stacked ``safe_mask`` is then:
+
+        .. math::
+            \overline{\epsilon_k} = \epsilon_{1k} OR \epsilon_{2k}
+
+        Please refer to the `~gammapy.irf.IRFStacker` for the description
+        of how the IRFs are stacked.
+
+        Parameters
+        ----------
+        other : `~gammapy.spectrum.SpectrumDataset`
+            the dataset to stack to the current one
+        """
+        if not isinstance(other, SpectrumDataset):
+            raise TypeError("Incompatible types for SpectrumDataset stacking")
+
+        if self.counts is not None:
+            self.counts.data[~self.mask_safe] = 0
+            self.counts.data[other.mask_safe] += other.counts.data[other.mask_safe]
+
+        if self.background is not None:
+            self.background.data[~self.mask_safe] = 0
+            self.background.data[other.mask_safe] += other.background.data[
+                other.mask_safe
+            ]
+
+        irf_stacker = IRFStacker(
+            list_aeff=[self.aeff, other.aeff],
+            list_livetime=[self.livetime, other.livetime],
+            list_edisp=[self.edisp, other.edisp],
+            list_low_threshold=[self.energy_range[0], other.energy_range[0]],
+            list_high_threshold=[self.energy_range[1], other.energy_range[1]],
+        )
+
+        if self.aeff is not None:
+            irf_stacker.stack_aeff()
+            if self.edisp is not None:
+                irf_stacker.stack_edisp()
+                self.edisp = irf_stacker.stacked_edisp
+            self.aeff = irf_stacker.stacked_aeff
+
+        self.mask_safe = np.logical_or(self.mask_safe, other.mask_safe)
+
+        if self.gti is not None:
+            self.gti = self.gti.stack(other.gti).union()
+
+        # TODO: for the moment, since dead time is not accounted for, livetime cannot be the sum
+        # of GTIs
+        self.livetime += other.livetime
+
 
 class SpectrumDatasetOnOff(SpectrumDataset):
     """Spectrum dataset for on-off likelihood fitting.
@@ -370,7 +494,7 @@ class SpectrumDatasetOnOff(SpectrumDataset):
 
     Parameters
     ----------
-    model : `~gammapy.spectrum.models.SpectralModel`
+    model : `~gammapy.modeling.models.SpectralModel`
         Fit model
     counts : `~gammapy.spectrum.CountsSpectrum`
         ON Counts spectrum
@@ -390,15 +514,14 @@ class SpectrumDatasetOnOff(SpectrumDataset):
         Relative background efficiency in the on region.
     acceptance_off : `~numpy.array` or float
         Relative background efficiency in the off region.
-    obs_id : int or list of int
-        Observation id(s) corresponding to the (stacked) dataset.
-    gti : '~gammapy.data.gti.GTI'
+    name : str
+        Name of the dataset.
+    gti : `~gammapy.data.GTI`
         GTI of the observation or union of GTI if it is a stacked observation
 
     See Also
     --------
     SpectrumDataset, FluxPointsDataset, MapDataset
-
     """
 
     likelihood_type = "wstat"
@@ -415,7 +538,7 @@ class SpectrumDatasetOnOff(SpectrumDataset):
         mask_fit=None,
         acceptance=None,
         acceptance_off=None,
-        obs_id=None,
+        name="",
         gti=None,
     ):
 
@@ -440,13 +563,9 @@ class SpectrumDatasetOnOff(SpectrumDataset):
 
         self.acceptance = acceptance
         self.acceptance_off = acceptance_off
-        self.obs_id = obs_id
+        self.name = name
         self.gti = gti
 
-    def __repr__(self):
-        str_ = self.__class__.__name__
-        return str_
-
     def __str__(self):
         str_ = super().__str__()
 
@@ -489,8 +608,7 @@ class SpectrumDatasetOnOff(SpectrumDataset):
     def fake(self, background_model, random_state="random-seed"):
         """Simulate fake counts for the current model and reduced irfs.
 
-         This method overwrites the counts and off counts defined on the dataset object.
-
+        This method overwrites the counts and off counts defined on the dataset object.
 
         Parameters
         ----------
@@ -498,8 +616,8 @@ class SpectrumDatasetOnOff(SpectrumDataset):
             BackgroundModel. In the future will be part of the SpectrumDataset Class.
             For the moment, a CountSpectrum.
         random_state : {int, 'random-seed', 'global-rng', `~numpy.random.RandomState`}
-                Defines random number generator initialisation.
-                Passed to `~gammapy.utils.random.get_random_state`.
+            Defines random number generator initialisation.
+            Passed to `~gammapy.utils.random.get_random_state`.
         """
         random_state = get_random_state(random_state)
 
@@ -515,6 +633,51 @@ class SpectrumDatasetOnOff(SpectrumDataset):
         npred_off.data = random_state.poisson(npred_off.data)
         self.counts_off = npred_off
 
+    @classmethod
+    def create(cls, e_reco, e_true=None, reference_time="2000-01-01"):
+        """Create empty SpectrumDatasetOnOff.
+
+        Empty containers are created with the correct geometry.
+        counts, counts_off and aeff are zero and edisp is diagonal.
+
+        The safe_mask is set to False in every bin.
+
+        Parameters
+        ----------
+        e_reco : `~astropy.units.Quantity`
+            edges of counts vector
+        e_true : `~astropy.units.Quantity`
+            edges of effective area table. If not set use reco energy values. Default : None
+        reference_time : `~astropy.time.Time`
+            reference time of the dataset, Default is "2000-01-01"
+        """
+        if e_true is None:
+            e_true = e_reco
+
+        counts = CountsSpectrum(e_reco[:-1], e_reco[1:])
+        counts_off = CountsSpectrum(e_reco[:-1], e_reco[1:])
+        aeff = EffectiveAreaTable(
+            e_true[:-1], e_true[1:], np.zeros(e_true[:-1].shape) * u.m ** 2
+        )
+        edisp = EnergyDispersion.from_diagonal_response(e_true, e_reco)
+        mask_safe = np.zeros_like(counts.data, "bool")
+        gti = GTI.create(u.Quantity([], "s"), u.Quantity([], "s"), reference_time)
+        livetime = gti.time_sum
+        acceptance = np.ones_like(counts.data, int)
+        acceptance_off = np.ones_like(counts.data, int)
+
+        return SpectrumDatasetOnOff(
+            counts=counts,
+            counts_off=counts_off,
+            aeff=aeff,
+            edisp=edisp,
+            mask_safe=mask_safe,
+            acceptance=acceptance,
+            acceptance_off=acceptance_off,
+            livetime=livetime,
+            gti=gti,
+        )
+
     @classmethod
     def read(cls, filename):
         """Read from file
@@ -531,6 +694,147 @@ class SpectrumDatasetOnOff(SpectrumDataset):
             "To read from an OGIP fits file use SpectrumDatasetOnOff.from_ogip_files."
         )
 
+    def _is_stackable(self):
+        """Check if the Dataset contains enough information to be stacked"""
+        if (
+            self.acceptance_off is None
+            or self.acceptance is None
+            or self.counts_off is None
+        ):
+            return False
+        else:
+            return True
+
+    def stack(self, other):
+        r"""Stack this dataset with another one.
+
+        Safe mask is applied to compute the stacked counts vector.
+        Counts outside each dataset safe mask are lost.
+
+        Stacking is performed in-place.
+
+        The stacking of 2 datasets is implemented as follows.
+        Here, :math:`k`  denotes a bin in reconstructed energy and :math:`j = {1,2}` is the dataset number
+
+        The ``mask_safe`` of each dataset is defined as:
+
+        .. math::
+            \epsilon_{jk} =\left\{\begin{array}{cl} 1, &
+            \mbox{if k is inside the energy thresholds}\\ 0, &
+            \mbox{otherwise} \end{array}\right.
+
+        Then the total ``counts`` and ``counts_off`` are computed according to:
+
+        .. math::
+            \overline{\mathrm{n_{on}}}_k =  \mathrm{n_{on}}_{1k} \cdot \epsilon_{1k} +
+            \mathrm{n_{on}}_{2k} \cdot \epsilon_{2k}
+
+            \overline{\mathrm{n_{off}}}_k = \mathrm{n_{off}}_{1k} \cdot \epsilon_{1k} +
+            \mathrm{n_{off}}_{2k} \cdot \epsilon_{2k}
+
+        The stacked ``safe_mask`` is then:
+
+        .. math::
+            \overline{\epsilon_k} = \epsilon_{1k} OR \epsilon_{2k}
+
+        In each energy bin :math:`k`, the count excess is computed taking into account the ON ``acceptance``,
+        :math:`a_{on}_k` and the OFF one: ``acceptance_off``, :math:`a_{off}_k`. They define
+        the :math:`\alpha_k=a_{on}_k/a_{off}_k` factors such that :math:`n_{ex}_k = n_{on}_k - \alpha_k n_{off}_k`.
+        We define the stacked value of :math:`\overline{{a}_{on}}_k = 1` so that:
+
+        .. math::
+            \overline{{a}_{off}}_k = \frac{\overline{\mathrm {n_{off}}}}{\alpha_{1k} \cdot
+            \mathrm{n_{off}}_{1k} \cdot \epsilon_{1k} + \alpha_{2k} \cdot \mathrm{n_{off}}_{2k} \cdot \epsilon_{2k}}
+
+        Please refer to the `~gammapy.irf.IRFStacker` for the description
+        of how the IRFs are stacked.
+
+        Parameters
+        ----------
+        other : `~gammapy.spectrum.SpectrumDatasetOnOff`
+            the dataset to stack to the current one
+
+        Examples
+        --------
+        >>> from gammapy.spectrum import SpectrumDatasetOnOff
+        >>> obs_ids = [23523, 23526, 23559, 23592]
+        >>> datasets = []
+        >>> for obs in obs_ids:
+        >>>     filename = "$GAMMAPY_DATA/joint-crab/spectra/hess/pha_obs{}.fits"
+        >>>     ds = SpectrumDatasetOnOff.from_ogip_files(filename.format(obs))
+        >>>     datasets.append(ds)
+        >>> stacked = datasets[0]
+        >>> for ds in datasets[1:]:
+        >>>     stacked.stack(ds)
+        >>> print(stacked.livetime)
+        6313.8116406202325 s
+        """
+        if not isinstance(other, SpectrumDatasetOnOff):
+            raise TypeError("Incompatible types for SpectrumDatasetOnOff stacking")
+
+        # We assume here that counts_off, acceptance and acceptance_off are well defined.
+        if not self._is_stackable() or not other._is_stackable():
+            raise ValueError("Cannot stack incomplete SpectrumDatsetOnOff.")
+
+        total_off = np.zeros_like(self.counts_off.data, dtype=float)
+        total_alpha = np.zeros_like(self.counts_off.data, dtype=float)
+
+        total_off[self.mask_safe] += self.counts_off.data[self.mask_safe]
+        total_off[other.mask_safe] += other.counts_off.data[other.mask_safe]
+        total_alpha[self.mask_safe] += (self.alpha * self.counts_off)[self.mask_safe]
+        total_alpha[other.mask_safe] += (other.alpha * other.counts_off)[
+            other.mask_safe
+        ]
+
+        with np.errstate(divide="ignore", invalid="ignore"):
+            acceptance_off = total_off / total_alpha
+            average_alpha = total_alpha.sum() / total_off.sum()
+
+        acceptance = np.ones_like(self.counts_off.data, dtype=float)
+        is_zero = total_off == 0
+        # For the bins where the stacked OFF counts equal 0, the alpha value is performed by weighting on the total
+        # OFF counts of each run
+        acceptance_off[is_zero] = 1 / average_alpha
+
+        self.acceptance = acceptance
+        self.acceptance_off = acceptance_off
+
+        if self.counts is not None:
+            self.counts.data[~self.mask_safe] = 0
+            self.counts.data[other.mask_safe] += other.counts.data[other.mask_safe]
+
+        if self.counts_off is not None:
+            self.counts_off.data[~self.mask_safe] = 0
+            self.counts_off.data[other.mask_safe] += other.counts_off.data[
+                other.mask_safe
+            ]
+
+        # We need to put this first to avoid later modifications of the energy ranges
+        irf_stacker = IRFStacker(
+            list_aeff=[self.aeff, other.aeff],
+            list_livetime=[self.livetime, other.livetime],
+            list_edisp=[self.edisp, other.edisp],
+            list_low_threshold=[self.energy_range[0], other.energy_range[0]],
+            list_high_threshold=[self.energy_range[1], other.energy_range[1]],
+        )
+
+        if self.aeff is not None:
+            if self.edisp is not None:
+                irf_stacker.stack_edisp()
+                self.edisp = irf_stacker.stacked_edisp
+            irf_stacker.stack_aeff()
+            self.aeff = irf_stacker.stacked_aeff
+
+        # We need to put this last not to modify the self.energy_range for previous calculations
+        self.mask_safe = np.logical_or(self.mask_safe, other.mask_safe)
+
+        if self.gti is not None:
+            self.gti = self.gti.stack(other.gti).union()
+
+        # TODO: for the moment, since dead time is not accounted for, livetime cannot be the sum
+        # of GTIs
+        self.livetime += other.livetime
+
     def peek(self, figsize=(10, 10)):
         """Quick-look summary plots."""
         import matplotlib.pyplot as plt
@@ -568,8 +872,10 @@ class SpectrumDatasetOnOff(SpectrumDataset):
         ax3.axis("off")
 
         if self.counts_off is not None:
-            stats = ObservationStats(**self._info_dict(in_safe_energy_range=True))
-            ax3.text(0, 0.2, "{}".format(stats), fontsize=12)
+            info = self._info_dict(in_safe_energy_range=True)
+            info["obs_id"] = info.pop("name")
+            stats = ObservationStats(**info)
+            ax3.text(0, 0.2, f"{stats}", fontsize=12)
 
         ax4.set_title("Energy Dispersion")
         if self.edisp is not None:
@@ -598,10 +904,7 @@ class SpectrumDatasetOnOff(SpectrumDataset):
         outdir = Path.cwd() if outdir is None else make_path(outdir)
         outdir.mkdir(exist_ok=True, parents=True)
 
-        if isinstance(self.obs_id, list):
-            phafile = "pha_stacked.fits"
-        else:
-            phafile = "pha_obs{}.fits".format(self.obs_id)
+        phafile = f"pha_obs{self.name}.fits"
 
         bkgfile = phafile.replace("pha", "bkg")
         arffile = phafile.replace("pha", "arf")
@@ -658,23 +961,23 @@ class SpectrumDatasetOnOff(SpectrumDataset):
 
     def _ogip_meta(self):
         """Meta info for the OGIP data format"""
-        meta = OrderedDict()
-        meta["name"] = "SPECTRUM"
-        meta["hduclass"] = "OGIP"
-        meta["hduclas1"] = "SPECTRUM"
-        meta["corrscal"] = ""
-        meta["chantype"] = "PHA"
-        meta["detchans"] = self.counts.energy.nbin
-        meta["filter"] = "None"
-        meta["corrfile"] = ""
-        meta["poisserr"] = True
-        meta["hduclas3"] = "COUNT"
-        meta["hduclas4"] = "TYPE:1"
-        meta["lo_thres"] = self.energy_range[0].to_value("TeV")
-        meta["hi_thres"] = self.energy_range[1].to_value("TeV")
-        meta["exposure"] = self.livetime.to_value("s")
-        meta["obs_id"] = self.obs_id
-        return meta
+        return {
+            "name": "SPECTRUM",
+            "hduclass": "OGIP",
+            "hduclas1": "SPECTRUM",
+            "corrscal": "",
+            "chantype": "PHA",
+            "detchans": self.counts.energy.nbin,
+            "filter": "None",
+            "corrfile": "",
+            "poisserr": True,
+            "hduclas3": "COUNT",
+            "hduclas4": "TYPE:1",
+            "lo_thres": self.energy_range[0].to_value("TeV"),
+            "hi_thres": self.energy_range[1].to_value("TeV"),
+            "exposure": self.livetime.to_value("s"),
+            "obs_id": self.name,
+        }
 
     @classmethod
     def from_ogip_files(cls, filename):
@@ -703,7 +1006,7 @@ class SpectrumDatasetOnOff(SpectrumDataset):
         try:
             rmffile = phafile.replace("pha", "rmf")
             energy_dispersion = EnergyDispersion.read(str(dirname / rmffile))
-        except IOError:
+        except OSError:
             # TODO : Add logger and echo warning
             energy_dispersion = None
 
@@ -720,7 +1023,7 @@ class SpectrumDatasetOnOff(SpectrumDataset):
                 )
 
                 acceptance_off = data_bkg["backscal"]
-        except IOError:
+        except OSError:
             # TODO : Add logger and echo warning
             counts_off, acceptance_off = None, None
 
@@ -738,7 +1041,7 @@ class SpectrumDatasetOnOff(SpectrumDataset):
             mask_safe=mask_safe,
             acceptance=data["backscal"],
             acceptance_off=acceptance_off,
-            obs_id=data["obs_id"],
+            name=str(data["obs_id"]),
         )
 
     # TODO: decide on a design for dataset info tables / dicts and make it part
@@ -760,7 +1063,7 @@ class SpectrumDatasetOnOff(SpectrumDataset):
             info["a_off"] = 1
 
         info["livetime"] = self.livetime
-        info["obs_id"] = self.obs_id
+        info["name"] = self.name
         return info
 
 
@@ -794,214 +1097,3 @@ def _read_ogip_hdulist(hdulist, hdu1="SPECTRUM", hdu2="EBOUNDS"):
         obs_id=counts_table.meta["OBS_ID"],
         is_bkg=False,
     )
-
-
-class SpectrumDatasetOnOffStacker:
-    r"""Stack a list of homogeneous datasets.
-
-    The stacking of :math:`j` datasets is implemented as follows.
-    :math:`k` and :math:`l` denote a bin in reconstructed and true energy,
-    respectively.
-
-    .. math::
-        \epsilon_{jk} =\left\{\begin{array}{cl} 1, & \mbox{if
-            bin k is inside the energy thresholds}\\ 0, & \mbox{otherwise} \end{array}\right.
-
-        \overline{\mathrm{n_{on}}}_k = \sum_{j} \mathrm{n_{on}}_{jk} \cdot
-            \epsilon_{jk}
-
-        \overline{\mathrm{n_{off}}}_k = \sum_{j} \mathrm{n_{off}}_{jk} \cdot
-            \epsilon_{jk}
-
-        \overline{\alpha}_k =
-        \frac{\overline{{b_{on}}}_k}{\overline{{b_{off}}}_k}
-
-        \overline{{b}_{on}}_k = 1
-
-        \overline{{b}_{off}}_k = \frac{1}{\sum_{j}\alpha_{jk} \cdot
-            \mathrm{n_{off}}_{jk} \cdot \epsilon_{jk}} \cdot \overline{\mathrm {n_{off}}}
-
-    Please refer to the `~gammapy.irf.IRFStacker` for the description
-    of how the IRFs are stacked.
-
-    Parameters
-    ----------
-    obs_list : list of `~gammapy.spectrum.SpectrumDatasetOnOff`
-        Observations to stack
-
-    Examples
-    --------
-    >>> from gammapy.spectrum import SpectrumDatasetOnOff, SpectrumDatasetOnOffStacker
-    >>> obs_ids = [23523, 23526, 23559, 23592]
-    >>> datasets = []
-    >>> for obs in obs_ids:
-    >>>     filename = "$GAMMAPY_DATA/joint-crab/spectra/hess/pha_obs{}.fits"
-    >>>     ds = SpectrumDatasetOnOff.from_ogip_files(filename.format(obs))
-    >>>     datasets.append(ds)
-    >>> obs_stacker = SpectrumDatasetOnOffStacker(datasets)
-    >>> stacked = obs_stacker.run()
-    >>> print(stacked.livetime)
-    6313.8116406202325 s
-    """
-
-    def __init__(self, obs_list):
-        self.obs_list = obs_list
-        self.stacked_on_vector = None
-        self.stacked_off_vector = None
-        self.stacked_aeff = None
-        self.stacked_edisp = None
-        self.stacked_bkscal_on = None
-        self.stacked_bkscal_off = None
-        self.stacked_obs = None
-        self.stacked_gti = None
-
-    def __str__(self):
-        ss = self.__class__.__name__
-        ss += "\n{}".format(self.obs_list)
-        return ss
-
-    def run(self):
-        """Run all steps in the correct order."""
-        self.stack_counts_vectors()
-        self.stack_aeff()
-        self.stack_edisp()
-        self.stack_gti()
-        self.stack_obs()
-        return self.stacked_obs
-
-    def stack_counts_vectors(self):
-        """Stack on and off vectors."""
-        self.stack_on_vector()
-        self.stack_off_vector()
-        self.stack_backscal()
-        self.setup_counts_vectors()
-
-    def stack_on_vector(self):
-        """Stack the on count vector."""
-        on_vector_list = [o.counts for o in self.obs_list]
-        self.stacked_on_vector = self.stack_counts_spectrum(on_vector_list)
-
-    def stack_off_vector(self):
-        """Stack the off count vector."""
-        off_vector_list = [o.counts_off for o in self.obs_list]
-        self.stacked_off_vector = self.stack_counts_spectrum(off_vector_list)
-
-    def stack_counts_spectrum(self, counts_spectrum_list):
-        """Stack `~gammapy.spectrum.CountsSpectrum`.
-
-        * Bins outside the safe energy range are set to 0
-        * Attributes are set to None.
-        * The quality vector of the observations are combined with a logical or,
-          such that the low (high) threshold of the stacked obs is the minimum
-          low (maximum high) threshold of the observation list to be stacked.
-        """
-        template = counts_spectrum_list[0].copy()
-        energy = template.energy
-        stacked_data = np.zeros(energy.nbin)
-        stacked_quality = np.ones(energy.nbin)
-        for spec, obs in zip(counts_spectrum_list, self.obs_list):
-            stacked_data[obs.mask_safe] += spec.data[obs.mask_safe]
-            temp = np.logical_and(stacked_quality, ~obs.mask_safe)
-            stacked_quality = np.array(temp, dtype=int)
-
-        self.stacked_quality = stacked_quality
-        return CountsSpectrum(
-            data=stacked_data, energy_lo=energy.edges[:-1], energy_hi=energy.edges[1:]
-        )
-
-    def stack_backscal(self):
-        """Stack ``backscal`` for on and off vector."""
-        nbins = self.obs_list[0].counts.energy.nbin
-        bkscal_on = np.ones(nbins)
-        bkscal_off = np.zeros(nbins)
-
-        alpha_sum = 0.0
-
-        for obs in self.obs_list:
-            bkscal_off[obs.mask_safe] += (obs.alpha * obs.counts_off.data)[
-                obs.mask_safe
-            ]
-            alpha_sum += (obs.alpha * obs.counts_off.data)[obs.mask_safe].sum()
-
-        with np.errstate(divide="ignore", invalid="ignore"):
-            stacked_bkscal_off = self.stacked_off_vector.data / bkscal_off
-            alpha_average = (
-                alpha_sum / self.stacked_off_vector.data[obs.mask_safe].sum()
-            )
-
-        # there should be no nan values in backscal_on or backscal_off
-        # this leads to problems when fitting the data
-        # use 1 for backscale of on_vector and 1 / alpha_average for backscale of off_vector
-        alpha_correction = 1
-        idx = np.where(self.stacked_off_vector.data == 0)[0]
-        bkscal_on[idx] = alpha_correction
-        # For the bins where the stacked OFF counts equal 0, the alpha value is performed by weighting on the total
-        # OFF counts of each run
-        stacked_bkscal_off[idx] = alpha_correction / alpha_average
-
-        self.stacked_bkscal_on = bkscal_on
-        self.stacked_bkscal_off = stacked_bkscal_off
-
-    def setup_counts_vectors(self):
-        """Add correct attributes to stacked counts vectors."""
-        livetimes = [obs.livetime.to_value("s") for obs in self.obs_list]
-        self.total_livetime = u.Quantity(np.sum(livetimes), "s")
-
-        self.stacked_on_vector.livetime = self.total_livetime
-        self.stacked_off_vector.livetime = self.total_livetime
-        self.stacked_on_vector.backscal = self.stacked_bkscal_on
-        self.stacked_off_vector.backscal = self.stacked_bkscal_off
-
-    def stack_aeff(self):
-        """Stack effective areas (weighted by livetime).
-
-        Calls `gammapy.irf.IRFStacker.stack_aeff`.
-        """
-        irf_stacker = IRFStacker(
-            list_aeff=[obs.aeff for obs in self.obs_list],
-            list_livetime=[obs.livetime for obs in self.obs_list],
-        )
-        irf_stacker.stack_aeff()
-        self.stacked_aeff = irf_stacker.stacked_aeff
-
-    def stack_edisp(self):
-        """Stack energy dispersion (weighted by exposure).
-
-        Calls `~gammapy.irf.IRFStacker.stack_edisp`
-        """
-        irf_stacker = IRFStacker(
-            list_aeff=[obs.aeff for obs in self.obs_list],
-            list_livetime=[obs.livetime for obs in self.obs_list],
-            list_edisp=[obs.edisp for obs in self.obs_list],
-            list_low_threshold=[obs.energy_range[0] for obs in self.obs_list],
-            list_high_threshold=[obs.energy_range[1] for obs in self.obs_list],
-        )
-        irf_stacker.stack_edisp()
-        self.stacked_edisp = irf_stacker.stacked_edisp
-
-    def stack_gti(self):
-        """Stack GTI
-        """
-        first_gti = self.obs_list[0].gti
-        if first_gti is None:
-            self.stacked_gti = None
-        else:
-            stack_gti = first_gti.copy()
-            for obs in self.obs_list[1:]:
-                stack_gti = stack_gti.stack(obs.gti)
-            self.stacked_gti = stack_gti.union()
-
-    def stack_obs(self):
-        """Create stacked `~gammapy.spectrum.SpectrumDatasetOnOff`."""
-        self.stacked_obs = SpectrumDatasetOnOff(
-            counts=self.stacked_on_vector,
-            counts_off=self.stacked_off_vector,
-            aeff=self.stacked_aeff,
-            edisp=self.stacked_edisp,
-            livetime=self.total_livetime,
-            mask_safe=np.logical_not(self.stacked_quality),
-            acceptance=self.stacked_on_vector.backscal,
-            acceptance_off=self.stacked_off_vector.backscal,
-            obs_id=[obs.obs_id for obs in self.obs_list],
-            gti=self.stacked_gti,
-        )
